#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "fitsio.h"
#include "CmdLine.h"
#include<vector>
#include "fitslib.h"
using namespace std;
#include <sstream>

#ifndef __iostream
#define  __iostream
//#include <iostream.h>
#include <iostream>
#endif
#include<fstream> 

int read_cat(string catfile,vector<double> & lvin,vector<double> & lvout,vector<int> & lxmin,vector<int> & lxmax,vector<int> & lymin,vector<int> & lymax);
int read_cat(string catfile,vector<double> & lvin,vector<double> & lvout);
int usage();
int filter_fits(fits & infits,vector<double> & lvin,vector<double> & lvout,vector<int> & lxmin,vector<int> & lxmax,vector<int> & lymin,vector<int> & lymax);
int filter_fits(fits & infits,vector<double> & lvin,vector<double> & lvout,int xmin,int xmax,int ymin,int ymax);

int main(int argc, char *argv[]) {

	CCmdLine cmdLine;
	
	// Parse it
	if (cmdLine.SplitLine(argc,argv) < 1) {
		//ASSERT(0);
		exit(-1);	
	}
	
	// Help case
	if (cmdLine.HasSwitch("-h")) {
		usage();
		exit(0);	
	}
	
	// Get the required arguments
	string in,out;
	try {
		in = cmdLine.GetArgument("-in",0);
	} 
	catch(...) {
		//ASSERT(0);
		cout << "Error ... aruments missing ... \n";
		usage();
		exit(-1);	
	}
	
	// Test if empty arguments
	if (in.size()==0) {
		usage();
		exit(-1);
	}
	
	//cout << "Infile : " << in << "\n";
	
	// open infits
	fits infits(in.c_str());
	std::ostringstream s,s1;
	string naxis1,naxis2;
	s << infits.get_naxis1();
	naxis1 = s.str();
	s1 << infits.get_naxis2();
	naxis2 = s1.str();
	
	
	// optional arguments
	bool rangeSwitch = cmdLine.HasSwitch("-range");
	
	int xmin = atoi(cmdLine.GetSafeArgument("-x",0,"1").c_str());
	int xmax = atoi(cmdLine.GetSafeArgument("-X",0,naxis1.c_str()).c_str());
	int ymin = atoi(cmdLine.GetSafeArgument("-y",0,"1").c_str());
	int ymax = atoi(cmdLine.GetSafeArgument("-Y",0,naxis2.c_str()).c_str());
	//cout << "ranges " << xmin << " " << xmax << " " << ymin << " " << ymax << "\n";
	
	string cat = cmdLine.GetSafeArgument("-cat",0,"");
	double vin = atof(cmdLine.GetSafeArgument("-vin",0,"-1e60").c_str());
	double vout = atof(cmdLine.GetSafeArgument("-vout",0,"-1e60").c_str());
	//cout << "vin/vout " << vin << " " << vout << "\n";
		     
	// It catalog present ? 
	if (cat != "") {
		vector<double> lvin,lvout;
		vector<int> lxmin,lxmax,lymin,lymax;
		//cout << "Reading cat ... \n"; 
		if (rangeSwitch) {
		  cout << "range defined \n";
			read_cat(cat,lvin,lvout,lxmin,lxmax,lymin,lymax);
			filter_fits(infits,lvin,lvout,lxmin,lxmax,lymin,lymax);
		} else {
		  cout << "range NOT defined \n";
			read_cat(cat,lvin,lvout);
			filter_fits(infits,lvin,lvout,xmin,xmax,ymin,ymax);
		}    
	// Use direct cammand line arguments
	} else {
		if (vin == -1e60 || vout == -1e60) {
			usage(); 
			exit(-1);	
		} else { 
		  //cout << "xminnn " << xmin << " xmax " << xmax << "\n";
		  //cout << "yminnn " << ymin << " ymax " << ymax << "\n";
			vector<int> xrange,yrange;
			xrange.push_back(xmin);
			xrange.push_back(xmax);
			yrange.push_back(ymin);
			yrange.push_back(ymax);
			
			infits.store_pix(xrange,yrange);
			//cout << "xyminmax1 " << vin << " " << vout << " " << xmin << " " << xmax << " " << ymin << " " << ymax << "\n"; 
			infits.store_filter(vin,vout,xmin,xmax,ymin,ymax); 
			//cout << "xyminmax2 " << xmin << " " << xmax << " " << ymin << " " << ymax << "\n"; 
			infits.write_subset(); 
		}
	}
	
	   
	
	return 0;
}

int filter_fits(fits & infits,vector<double> & lvin,vector<double> & lvout,vector<int> & lxmin,vector<int> & lxmax,vector<int> & lymin,vector<int> & lymax) {
	infits.store_pix();
	
	int i;
	for(i=0;i<(int)lvin.size();i++) {
	  //cout << "xyminmax " << lvin[i] << " " << lvout[i] << " " << lxmin[i] << " " << lxmax[i] << " " << lymin[i] << " " << lymax[i] << "\n";
		infits.store_filter(lvin[i],lvout[i],lxmin[i],lxmax[i],lymin[i],lymax[i]);
	}
	infits.write_subset();
	return 0;
}     

int filter_fits(fits & infits,vector<double> & lvin,vector<double> & lvout,int xmin,int xmax,int ymin,int ymax) {
	infits.store_pix();
	
	int i;
	for(i=0;i<(int)lvin.size();i++) {
	  //cout << "xyminmax " << lvin[i] << " " << lvout[i] << " " << xmin << " " << xmax << " " << ymin << " " << ymax << "\n";
		infits.store_filter(lvin[i],lvout[i],xmin,xmax,ymin,ymax);
	}
	infits.write_subset();
	return 0;	
} 

int read_cat(string catfile,vector<double> & lvin,vector<double> & lvout,vector<int> & lxmin,vector<int> & lxmax,vector<int> & lymin,vector<int> & lymax) {
	
	// Read the file
	ifstream file;
	file.open(catfile.c_str());
	vector<string> keys;	
	double vin,vout;
	int xmin,xmax,ymin,ymax;
	
	while (! file.eof() ) {
		vin = -1e60;
		file >> vin >> vout >> xmin >> xmax >> ymin >> ymax;	
		if (vin != -1e60) {
			lvin.push_back(vin);
			lvout.push_back(vout);
			lxmin.push_back(xmin);
			lxmax.push_back(xmax);
			lymin.push_back(ymin);
			lymax.push_back(ymax);
		}
	}
	file.close();

	return 0;	
}

int read_cat(string catfile,vector<double> & lvin,vector<double> & lvout) {

	
	// Read the file
	ifstream file;
	file.open(catfile.c_str());
	vector<string> keys;	
	double vin,vout;
	
	while (! file.eof() ) {
		vin = -1e60;
		file >> vin >> vout;
		if (vin != -1e60) {
			lvin.push_back(vin);
			lvout.push_back(vout);
			//cout << "incat " << vin << " " << vout << "\n";
		}
	}
	file.close();

	return 0;	
}

int usage (){
	cout << "Usage : getpix_val -in infits -vin valin -vout valout [-x xmin -X xmax -y ymin -Y ymax] ...\n";	
	cout << "      : getpix_val -in infits -cat cat -range  ...\n";	
	return 0;	
}
