/* History comments (TE):
   ======================

   05.12.2008:
   Changes in the headers which are included to make the code compile
   with g++ 4.3

   06.04.2008:
   Changes in the inclusion of header files for newer g++ conmpilers
*/  

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <cstdio>
#include <cstdlib>
#include <ctime>
#include <cmath>

#include "fitsio.h"
#include "rdf_fits.hpp" 

using namespace std;

/* ------------------------------------- 
 Filter the segmentation image by removing the objects in the catalogue
 catalogue format : NUMBER XMIN XMAX YMIN YMAX
 --------------------------------------- */

/*
int printerror( int status) {
    if (status)
    {
       fits_report_error(stderr, status); // print error report 

       exit( status );    // terminate the program, returning error status 
    }
    return 0;
}
*/

int main (int argc, char *argv[]) {
	   
	char image[80],cat[80],out[80];
	int n,x1,x2,y1,y2;
	float **imagef;
	
	sscanf(argv[1],"%s",image);
	sscanf(argv[2],"%s",cat);
  	sscanf(argv[3],"%s",out);
	
	// Read the catalogue
	cout << "read cat \n";
	vector<int> lnumber,lx1,lx2,ly1,ly2;
	ifstream file(cat);
	while(file){
  		file >> n >> x1 >> y1 >> x2 >> y2;
  		lnumber.push_back(n);
  		lx1.push_back(x1);
  		lx2.push_back(x2);
  		ly1.push_back(y1);
  		ly2.push_back(y2);
  		
  		//cout << n << " " << x1 << "  " << x2 << "  " << y1 << "  " << y2 << "\n"; 
  		if (file.eof()) break;
	}
 	file.close();
	
	// Read the image
	cout << "Read the image \n";
	int itmp=3; 

    int nxtmp = 0;
    int nytmp = 0;
    float xmin = 0;
    float xmax = 0;
    float ymin = 0;
    float ymax = 0;
	imagef=(float **)rdf_fits(argv[1],&nxtmp,&nytmp,&xmin,&xmax,&ymin,&ymax,itmp);
	
	
	// Size of the image
	fitsfile *fptr;
	int status=0;
  	if (fits_open_file(&fptr, argv[1] , READONLY, &status)) {
           printerror( status );           /* call printerror if error occurs */
  	}


  	/* read the NAXIS1 and NAXIS2 keyword to get image size */
  	long int naxes[2]; int nfound;
  	if ( fits_read_keys_lng(fptr, "NAXIS", 1, 2, naxes, &nfound, &status) ) {
   	    printerror( status );          
  	}
	int sizex = naxes[0];
	int sizey = naxes[1];
	
	// Loop on the objects
	cout << "Loop on the objects \n";
	vector<int>::iterator it; 
	int ii = -1;
	int k,l,xxmin,xxmax,yymin,yymax;
	for(it=lnumber.begin();it!=lnumber.end();it++) {
		ii++;
		
		xxmin = (int) lx1[ii]-1;
		xxmax = (int) lx2[ii]+1;
		yymin = (int) ly1[ii]-1;
		yymax = (int) ly2[ii]+1;
		if (xxmin < 0) {xmin =0;}
		if (yymin < 0) {ymin =0;}
		if (xxmax >= sizex) {xxmax = sizex -1;}
		if (yymax >= sizey) {yymax = sizey -1;}
		
		cout << "Object " << ii << " xmin " << xxmin << " xmax " << xxmax << " ymin " << yymin << " ymax" << yymax << "\n";
		
		for(k=xxmin;k<=xxmax;k++) {
			for(l=yymin;l<=yymax;l++) {
				//cout << "test " << imagef[l][k] << " and " << *it << "\n";
				
				if (fabs(imagef[l][k]-*it) < 0.1) {
					imagef[l][k] = 0;
				}		
			}
		}
			
	}
	
	// write output image
	cout << "Write image \n";
	write_fits_tab(imagef,sizex,sizey,out);
	
	cout << "free image \n";
	free_square(imagef,sizey,sizex);
}
