/* History comments (TE):
   ======================

   05.12.2008:
   Changes in the headers which are included to make the code compile
   with g++ 4.3

   06.04.2009:
   more changes in the header include files for new g++ compilers
*/  

#include <iostream>
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include "fitslib.h"
#include "fitsio.h"

using namespace std;

// constructor
fits::fits(string name) {
	filename = name;
	iomode = 1;
	status = 0;
	fits_open_file(&fptr,filename.c_str(),iomode, &status);
	
	string com;
	string key1("NAXIS1");
	string key2("NAXIS2");
	
	read_fits_key_int(key1,naxis1,com);
	read_fits_key_int(key2,naxis2,com);
}

// destructor
fits::~fits() {
	fits_close_file(fptr,&status);
}

int fits::change_iomode (int io) {
	iomode = io;
	return 0;
}
int fits::change_filename (string name) {
	filename = name;
	return 0;	
}

int fits::printerror( int status) {
    if (status)
    {
       fits_report_error(stderr, status); // print error report 

       exit( status );    // terminate the program, returning error status 
    }
    return 0;
}

// Reading the header
int fits::get_nkeys() {
	int keypos;
	if(fits_get_hdrpos(fptr, &nkeys, &keypos, &status)) 
		{printerror( status ); }
	return nkeys;
}

int fits::read_fits_line (string & key,string & line) {
	char * card;
	card = (char *) malloc (100*sizeof(char));
	
	char keyy[100];
	strcpy(keyy,key.c_str());
	if(fits_read_card(fptr, keyy, card, &status)) 
		{printerror( status ); }
	
	// Copy it to the output string
	line = card;
	free(card);
	return 0;
}
int fits::read_fits_key_dbl (string & key,double & value,string &comment) {
	
	char com[100];
	char keyy[100];
	strcpy(keyy,key.c_str());
	if ( fits_read_key(fptr, TDOUBLE, keyy, &value,com, &status))
         {printerror( status );}          
    comment = com;
	return 0;
}
int fits::read_fits_key_int (string & key,int & value,string &comment) {
	
	char com[100];
	char keyy[100];
	strcpy(keyy,key.c_str());
	if ( fits_read_key(fptr, TLONG, keyy, &value,com, &status))
         {printerror( status );}          
    comment = com;
	return 0;
}
int fits::read_fits_key_str (string & key,string & value,string &comment) {
	
	char com[100],val[100];
	char keyy[100];
	strcpy(keyy,key.c_str());
	if ( fits_read_keyword(fptr,  keyy, val,com, &status))
         {printerror( status );}    
    comment = com;
    value = val;
	return 0;
}
int fits::read_header () {
	int nkeys = get_nkeys();
	int kk;
	char key[100],val[100],com[100];
	string keyy,vall,comm;
	
	for (kk = 1; kk <= nkeys; kk++) { 
		if(fits_read_keyn(fptr,kk,key,val,com,&status))
			printerror( status );
		keyy = key;
		vall = val;
		comm = com;
		Lvalues.push_back(keyy);
		Mvalues[keyy] = vall; Mcomments[keyy] = comm;
	}
	return 0;
}
int fits::get_key(string key,string & res,string & com) {
	res = Mvalues[key];
	com = Mcomments[key];
	return 0;	
}

// Writing header
int fits::write_keys_num(vector<string> keys,vector<double> vals, vector<string> comments) {
	char key[80];
	char com[80];
	
	int ii = -1;
	vector<string>::iterator it;	
	for (it=keys.begin();it!=keys.end();it++) {
		ii++;	
		int decimals = 10;
		
		sscanf(keys[ii].c_str(),"%s",key);
		strcpy(com,comments[ii].c_str());
		float tmp = (float) vals[ii];
		if (ffukyf(fptr,key,tmp,decimals,com,&status)) 
			printerror( status );
	}
	return 0;
}
int fits::write_keys_str(vector<string> keys,vector<string> vals, vector<string> comments) {
	char key[80];
	char com[80];
	char val[80];
	int ii = -1;
	vector<string>::iterator it;	
	for (it=keys.begin();it!=keys.end();it++) {
		ii++;	
		sscanf(keys[ii].c_str(),"%s",key);
		sscanf(vals[ii].c_str(),"%s",val);
		strcpy(com,comments[ii].c_str());
		if (ffuky(fptr,TSTRING,key,val,com,&status)) 
			printerror( status );
	}
	
	return 0;
}
int fits::write_history (vector<string> & lstr) {
	char com[80];
	vector<string>::iterator it;	
	for (it=lstr.begin();it!=lstr.end();it++) {
		strcpy(com,it->c_str());
		if(fits_write_history(fptr,com,&status)) 
			printerror( status );
	}
	return 0;
}
int fits::write_comments (vector<string> & lstr) {
	char com[80];
	vector<string>::iterator it;	
	for (it=lstr.begin();it!=lstr.end();it++) {
		strcpy(com,it->c_str());
		if(fits_write_comment(fptr,com,&status)) 
			printerror( status );
	}
	return 0;
}

// Copy keywords
int fits::cphead_num(fits & fits_file,vector<string> & keys){
	vector<string> coms;
	vector<double> vals;
	vector<string>::iterator it;
	int ii = -1;
	
	// Read them
	string com;
	double val;
	for(it = keys.begin();it != keys.end();it++) {
		ii++;
		fits_file.read_fits_key_dbl(*it,val,com);
		coms.push_back(com);
		vals.push_back(val);
	}	
	
	// Write them
	write_keys_num(keys,vals,coms);
	
	return 0;	
}
int fits::cphead_str(fits & fits_file,vector<string> & keys){
	vector<string> coms;
	vector<string> vals;
	vector<string>::iterator it;
	int ii = -1;
	
	// Read them
	string com;
	string val;
	for(it = keys.begin();it != keys.end();it++) {
		ii++;
		fits_file.read_fits_key_str(*it,val,com);
		coms.push_back(com);
		vals.push_back(val);
	}	
	
	// Write them
	write_keys_str(keys,vals,coms);
	
	return 0;	
}

// Read pixels
int fits::read_px(vector<int> xrange,vector<int> yrange,vector<float> & px) {
	
	// Get images size
	
	int npix = (xrange[1]-xrange[0]+1)*(yrange[1]-yrange[0]+1);
	float nulval = -1;
	//long inc = 1;
	long ii;
	
	// Check range
	if (xrange[0] < 1) {exit(1);}
	if (yrange[0] < 1) {exit(1);}
	if (xrange[1] > naxis1) {exit(1);}
	if (yrange[1] > naxis2) {exit(1);}
	
	// Create buffer
	float *buff;
	buff = new(float [npix]);
	
	// Read the pixels
	long fpixel[2] = {(long)xrange[0],(long)yrange[0]};
	long lpixel[2] = {(long)xrange[1],(long)yrange[1]};
	long inc[2] = {1,1};
	
	if (fits_read_subset(fptr,TFLOAT, fpixel, lpixel, inc,&nulval,buff, NULL, &status))
		printerror( status );
	
	px.assign(npix,0.0);
	for(ii=0;ii<npix;ii++) {
		px[ii] = buff[ii];	
	}
	
	delete [] buff;
	return 0;	
}
int fits::read_px_sq(vector<int> xrange,vector<int> yrange,vector< vector<float> > & px) {
	
	// Get images size
	
	int npix = (xrange[1]-xrange[0]+1)*(yrange[1]-yrange[0]+1);
	float nulval = -1;
	long ii,jj;
	
	// Check range
	if (xrange[0] < 1) {exit(1);}
	if (yrange[0] < 1) {exit(1);}
	if (xrange[1] > naxis1) {exit(1);}
	if (yrange[1] > naxis2) {exit(1);}
	
	// Create buffer
	float *buff;
	buff = new(float [npix]);
	
	// Read the pixels
	long fpixel[2] = {(long)xrange[0],(long)yrange[0]};
	long lpixel[2] = {(long)xrange[1],(long)yrange[1]};
	long inc[2] = {1,1};
	int dec1 = xrange[1] - xrange[0];
	int dec2 = yrange[1] - yrange[0];	
	
	if (fits_read_subset(fptr,TFLOAT, fpixel, lpixel, inc,&nulval,buff, NULL, &status))
		printerror( status );
	
	// Transfer to the vectors
	vector<float> tmp;
	for(ii=0;ii<=dec1;ii++) {
		px.push_back(tmp);
		px[ii].assign(dec1+1,0.0);
	}
	
	int index = 0;
	for(jj=0;jj<=dec2;jj++) {
		//px.push_back(tmp);
		for(ii=0;ii<=dec1;ii++) {
			//px[ii].push_back(buff[index]);
			px[ii][jj] = buff[index];
			index++;	
		}
	}	
	
	delete [] buff;
	return 0;	
}
int fits::read_index_val(vector<int> xrange,vector<int> yrange,double val,vector< point > & index){
		
  //cout << "test \n";

	// Get images size
	
	int npix = (xrange[1]-xrange[0]+1)*(yrange[1]-yrange[0]+1);
	float nulval = -1;
	long ii,jj;
	
	// Check range
	if (xrange[0] < 1) {xrange[0] = 1;}
	if (yrange[0] < 1) {yrange[0] = 1;}
	if (xrange[1] > naxis1) {xrange[1] = naxis1;}
	if (yrange[1] > naxis2) {yrange[1] = naxis2;}
	
	//cout << "ranges " << xrange[0] << " " << xrange[1] << " " << yrange[0] << " " << yrange[1] << "\n";

	// Create buffer
	float *buff;
	buff = new(float [npix]);
	
	// Read the pixels
	long fpixel[2] = {(long)xrange[0],(long)yrange[0]};
	long lpixel[2] = {(long)xrange[1],(long)yrange[1]};
	long inc[2] = {1,1};
	int dec1 = xrange[1] - xrange[0];
	int dec2 = yrange[1] - yrange[0];	
	
	if (fits_read_subset(fptr,TFLOAT, fpixel, lpixel, inc,&nulval,buff, NULL, &status))
		printerror( status );
	
	// Transfer to the vectors
	int indexx = 0;
	point P;
	for(jj=0;jj<=dec2;jj++) {
		//px.push_back(tmp);
		for(ii=0;ii<=dec1;ii++) {
		  //cout << buff[indexx] << " ";
			if (buff[indexx] == val) {
				P.x = ii+xrange[0];
				P.y = jj+yrange[0];
				index.push_back(P);		
			}
			indexx++;	
		}
	}	
	
	delete [] buff;
	return 0;		
}
int fits::read_index_val(double val,vector< point > & index){
		
	vector<int> xrange,yrange;
	xrange.push_back(1);
	xrange.push_back(naxis1);
	yrange.push_back(1);
	yrange.push_back(naxis2);
	read_index_val(xrange,yrange,val,index);
	return 0;
}
int fits::read_index_lim(vector<int> xrange,vector<int> yrange,double val1,double val2,vector< point > & index){
		
	// Get images size
	
	int npix = (xrange[1]-xrange[0]+1)*(yrange[1]-yrange[0]+1);
	float nulval = -1;
	long ii,jj;
	
	// Check range
	if (xrange[0] < 1) {exit(1);}
	if (yrange[0] < 1) {exit(1);}
	if (xrange[1] > naxis1) {exit(1);}
	if (yrange[1] > naxis2) {exit(1);}
	
	// Create buffer
	float *buff;
	buff = new(float [npix]);
	
	// Read the pixels
	long fpixel[2] = {(long)xrange[0],(long)yrange[0]};
	long lpixel[2] = {(long)xrange[1],(long)yrange[1]};
	long inc[2] = {1,1};
	int dec1 = xrange[1] - xrange[0];
	int dec2 = yrange[1] - yrange[0];	
	
	if (fits_read_subset(fptr,TFLOAT, fpixel, lpixel, inc,&nulval,buff, NULL, &status))
		printerror( status );
	
	// Transfer to the vectors
	int indexx = 0;
	point P;
	for(jj=0;jj<=dec2;jj++) {
		//px.push_back(tmp);
		for(ii=0;ii<=dec1;ii++) {
			//cout << buff[indexx] << " ";
			if (buff[indexx] >= val1 && buff[indexx] <= val2) {
				P.x = ii+xrange[0];
				P.y = jj+yrange[0];
				index.push_back(P);		
			}
			indexx++;	
		}
	}	
	
	delete [] buff;
	return 0;		
}
int fits::read_index_lim_val(vector<int> xrange,vector<int> yrange,double val1,double val2,vector< point > & index, vector<double> & lval){
		
	// Get images size
	int npix = (xrange[1]-xrange[0]+1)*(yrange[1]-yrange[0]+1);
	float nulval = -1;
	long ii,jj;
	
	// Check range
	if (xrange[0] < 1) {exit(1);}
	if (yrange[0] < 1) {exit(1);}
	if (xrange[1] > naxis1) {exit(1);}
	if (yrange[1] > naxis2) {exit(1);}
	
	// Create buffer
	float *buff;
	buff = new(float [npix]);
	
	// Read the pixels
	long fpixel[2] = {(long)xrange[0],(long)yrange[0]};
	long lpixel[2] = {(long)xrange[1],(long)yrange[1]};
	long inc[2] = {1,1};
	int dec1 = xrange[1] - xrange[0];
	int dec2 = yrange[1] - yrange[0];	
	
	if (fits_read_subset(fptr,TFLOAT, fpixel, lpixel, inc,&nulval,buff, NULL, &status))
		printerror( status );
	
	// Transfer to the vectors
	int indexx = 0;
	point P;
	for(jj=0;jj<=dec2;jj++) {
		//px.push_back(tmp);
		for(ii=0;ii<=dec1;ii++) {
			//cout << buff[indexx] << " ";
			if (buff[indexx] >= val1 && buff[indexx] <= val2) {
				P.x = ii+xrange[0];
				P.y = jj+yrange[0];
				index.push_back(P);		
				lval.push_back(buff[indexx]);
			}
			indexx++;	
		}
	}	
	
	delete [] buff;
	return 0;		
}
int fits::read_index_lim(double val1,double val2,vector< point > & index){
		
	vector<int> xrange,yrange;
	xrange.push_back(1);
	xrange.push_back(naxis1);
	yrange.push_back(1);
	yrange.push_back(naxis2);
	read_index_lim(xrange,yrange,val1,val2,index);
	return 0;		
}
int fits::read_index_lim_val(double val1,double val2,vector< point > & index,vector<double> & lval){
		
	vector<int> xrange,yrange;
	xrange.push_back(1);
	xrange.push_back(naxis1);
	yrange.push_back(1);
	yrange.push_back(naxis2);
	read_index_lim_val(xrange,yrange,val1,val2,index,lval);
	return 0;		
}

int fits::store_pix() {
	vector<int> xrange,yrange;
	xrange.push_back(1);
	xrange.push_back(naxis1);
	yrange.push_back(1);
	yrange.push_back(naxis2);
	store_pix(xrange,yrange);
	
	return 0;
}
int fits::store_pix(vector<int> xrange,vector<int> yrange) {
	
	// Get images size
	int npix = (xrange[1]-xrange[0]+1)*(yrange[1]-yrange[0]+1);
	float nulval = -1;
	//long inc = 1;
	long ii;
	
	// Check range
	if (xrange[0] < 1) {exit(1);}
	if (yrange[0] < 1) {exit(1);}
	if (xrange[1] > naxis1) {exit(1);}
	if (yrange[1] > naxis2) {exit(1);}
	
	// Create buffer
	float *buff;
	buff = new(float [npix]);
	
	// Read the pixels
	long fpixel[2] = {(long)xrange[0],(long)yrange[0]};
	long lpixel[2] = {(long)xrange[1],(long)yrange[1]};
	long inc[2] = {1,1};
	
	if (fits_read_subset(fptr,TFLOAT, fpixel, lpixel, inc,&nulval,buff, NULL, &status))
		printerror( status );
		
	im.px.reserve(npix);
	im.xmin = xrange[0];
	im.ymin = yrange[0];
	im.xmax = xrange[1];
	im.ymax = yrange[1];
	im.dx = im.xmax - im.xmin + 1;
	im.dy = im.ymax - im.ymin + 1;
	
	im.px.assign(npix,0.0);
	for(ii=0;ii<npix;ii++) {
		//im.px.push_back(buff[ii]);	
		im.px[ii] = buff[ii];
		//cout << "px " << im.px[ii] << "\n";
	}
	
	delete [] buff;
	return 0;
	
}

int fits::index_store_val (double val, vector<point> & index) {
	int ii,jj;
	
	// Transfer to the vectors
	int indexx = 0;
	point P;
	for(jj=0;jj<im.dy;jj++) {
		for(ii=0;ii<im.dx;ii++) {
			//cout << buff[indexx] << " ";
			if (im.px[indexx] == val) {
				P.x = ii+im.xmin;
				P.y = jj+im.ymin;
				index.push_back(P);		
			}
			indexx++;	
		}
	}	
	
	return 0;
}

int fits::index_store_val_range (double val, vector<point> & index,int xmin,int xmax,int ymin,int ymax) {
	int ii,jj;
	
	int dx_ref = im.xmax - im.xmin + 1;
			
	float vin = (float)val;
	int iimin = xmin -im.xmin;
	int iimax = xmax -im.xmin;
	int jjmin = ymin -im.ymin;
	int jjmax = ymax -im.ymin;
	int dx = iimax - iimin + 1;
	int dy = jjmax - jjmin + 1;
	
	// Transfer to the vectors
	int indexx = 0;
	point P;
	for(jj=jjmin;jj<dy;jj++) {
		indexx = jjmin + jj * dx_ref; 
		for(ii=iimin;ii<dx;ii++) {
			//cout << buff[indexx] << " ";
			if (im.px[indexx] == vin) {
				P.x = ii+im.xmin;
				P.y = jj+im.ymin;
				index.push_back(P);		
			}
			indexx++;	
		}
	}	
	
	return 0;
}

int fits::index_store_lim (double val1,double val2, vector<point> & index) {
	int ii,jj;
	
	// Transfer to the vectors
	int indexx = 0;
	point P;
	for(jj=0;jj<im.dy;jj++) {
		for(ii=0;ii<im.dx;ii++) {
			//cout << buff[indexx] << " ";
			if (im.px[indexx] >= val1 && im.px[indexx] <= val2) {
				P.x = ii+im.xmin;
				P.y = jj+im.ymin;
				index.push_back(P);		
			}
			indexx++;	
		}
	}	
	
	return 0;
}

// WRITE PIXELS
int fits::write_subset(vector<int> xrange,vector<int> yrange,vector<float> value) {
	int ii;
	float * buff;
	
	// Check range
	if (xrange[0] < 1) {exit(1);}
	if (yrange[0] < 1) {exit(1);}
	if (xrange[1] > naxis1) {exit(1);}
	if (yrange[1] > naxis2) {exit(1);}
	
	// Transfer px to an array
	int dx = xrange[1] - xrange[0] + 1;
	int dy = yrange[1] - yrange[0] + 1;
	int npix = dx * dy;
	buff = new(float [npix]);
	
	// Check size of values compared to range
	int nelem = value.size();
	if (nelem != npix) {
		cout << "Problem ... \n";
		exit(1);	
	}
	
	long lpixel[2] = {(long)xrange[0],(long)yrange[0]};
	long fpixel[2] = {(long)xrange[1],(long)yrange[1]};
	
	for (ii=0;ii<npix;ii++) {
		buff[ii] = value[ii];	
	}
	
	if (fits_write_subset(fptr,TFLOAT,lpixel,fpixel,buff,&status))
		printerror( status );
	
	delete [] buff;
	return 0;
}
int fits::write_subset(vector<int> xrange,vector<int> yrange) {
	write_subset(xrange,yrange,im.px);
	return 0;
}
int fits::write_subset() {
	vector<int> xrange;
	vector<int> yrange;
	xrange.push_back(im.xmin);
	xrange.push_back(im.xmax);
	yrange.push_back(im.ymin);
	yrange.push_back(im.ymax);
	
	write_subset(xrange,yrange,im.px);
	return 0;
}
int fits::write_px(vector<int> xcoord,vector<int> ycoord,vector<float> val){
	long nel = 1;
	long pix[2];
	float value[1];
	
	long i;
	for (i=0;i<(long)xcoord.size();i++) {
		pix[0] = xcoord[i];
		pix[1] = ycoord[i];
		value[0] = val[i];	
		if (fits_write_pix(fptr,TFLOAT,pix,nel,value,&status))
			printerror( status );
	}
	
	return 0;	
}
int fits::write_single_px(int xcoord,int ycoord,float val){
	long nel = 1;
	long pix[2];
	float value[1];
	
	pix[0] = xcoord;
	pix[1] = ycoord;
	value[0] = val;	
	if (fits_write_pix(fptr,TFLOAT,pix,nel,value,&status))
		printerror( status ); 
	return 0;	
}



// FILTER IMAGE
int fits::store_filter (double vin2,double vout,int xmin,int xmax,int ymin,int ymax) {
	int ii,jj;
	
	//cout << "lim " << xmin << " " << xmax << " " << ymin << " " << ymax << "\n";
	//cout << "vin1 " << vin << " " << vout << "\n";
	
	if (xmax > im.xmax || ymax > im.ymax) {
		cout << "Size problems ... \n";
		exit(-1);
	}
		
	int dx_ref = im.xmax - im.xmin + 1;
			
	float vin = (float)vin2;
	int iimin = xmin -im.xmin;
	int iimax = xmax -im.xmin;
	int jjmin = ymin -im.ymin;
	int jjmax = ymax -im.ymin;
	int dx = iimax + 1;
	int dy = jjmax + 1;
	
	//cout << "jjmin " << jjmin << " " << dy << "\n"; 
	//cout << "iimin " << iimin << " " << dx << "\n"; 
	//cout << "vin1 " << vin << " " << vout << "\n";
	
	int indexx = 0;
	for(jj=jjmin;jj<dy;jj++) {
	  //indexx = jj * dx_ref;
		for(ii=iimin;ii<dx;ii++) {
		  indexx = (jj) * dx_ref + ii;
		  //cout << "val = " << im.px[indexx] << "\n";
			if (im.px[indexx] == vin) { 
				im.px[indexx] = vout;
			}	
		}
	}
	
	return 0;
}

int fits::store_filter_tot (double vin2,double vout,int xmin,int xmax,int ymin,int ymax) {
	int ii,jj;
	
	//cout << "lim " << xmin << " " << xmax << " " << ymin << " " << ymax << "\n";
	//cout << "vin1 " << vin2 << " " << vout << "\n";
	
		
			
	float vin = (float)vin2;
	int iimin = xmin - 1;
	int iimax = xmax - 1;
	int jjmin = ymin - 1;
	int jjmax = ymax - 1;
	int dx = iimax - iimin + 1;
	int dy = jjmax - jjmin + 1;
	
	cout << "lim " << jjmin << " " << dy << " " << iimin << " " << dx << "\n";
	
	int indexx = 0;
	for(jj=jjmin;jj<dy;jj++) {
		for(ii=iimin;ii<dx;ii++) {
			if (im.px[indexx] == vin) { 
				im.px[indexx] = vout;
			}	
			indexx++;
		}
	}
	
	return 0;
}

int fits::store_filter (double vin2,double vout) {
	int ii,jj;
	float vin = (float)vin2;
	
	int indexx = 0;
	for(jj=0;jj<im.dy;jj++) {
		for(ii=0;ii<im.dx;ii++) {
			if (im.px[indexx] == vin) { 
				im.px[indexx] = vout;
			}	
			indexx++;
		}
	}
	return 0;
}

