#ifndef CFITS_LIB_HPP_
#define CFITS_LIB_HPP_

#include <string>
#include <vector>
#include <map>
#include "fitsio.h"

using namespace std;

struct point {
	double x;
	double y;
};

struct subim {
		vector<float> px;
		int xmin,ymin;
		int xmax,ymax;
		int dx,dy;
};


class fits { 
	private:	
		fitsfile *fptr;
		string filename;
	
		int status;
		int iomode;
		
		// Image
		int naxis1,naxis2;
		
		// Header 
		int nkeys;
		int printerror( int status);
		map<string,string> Mvalues;
		map<string,string> Mcomments;
		vector<string> Lvalues;
		
		// Pixels
		subim im;
		
	public:
		fits(string name);
		~fits();
	
		int change_iomode (int io);
		int change_filename (string name);
		
		int get_naxis1 () {
			return naxis1;	
		}
		int get_naxis2 () {
			return naxis2;	
		}
		
		// header reading routines
		int get_nkeys();
		int read_fits_line(string & key,string & line);
		int read_fits_key_dbl(string & key,double & value,string &comment);
		int read_fits_key_int(string & key,int & value,string &comment);
		int read_fits_key_str (string & key,string & value,string &comment);
		int read_header();
		int get_key(string key,string & res,string & com);
		
		// Header writing routines
		int write_keys_num(vector<string> keys,vector<double> vals, vector<string> comments);
		int write_keys_str(vector<string> keys,vector<string> vals, vector<string> comments);
		int write_comments (vector<string> & lstr);
		int write_history (vector<string> & lstr);
			
		// Copy headers
		int cphead_num(fits & fits_file,vector<string> & keys);
		int cphead_str(fits & fits_file,vector<string> & keys);
			
		// Read pixels
		int read_px(vector<int> xrange,vector<int> yrange,vector<float> & px);
		int read_px_sq(vector<int> xrange,vector<int> yrange,vector< vector<float> > & px);
		int read_index_val(vector<int> xrange,vector<int> yrange,double val,vector< point > & index);
		int read_index_val(double val,vector< point > & index);
		int read_index_lim(vector<int> xrange,vector<int> yrange,double val1,double val2,vector< point > & index);
		int read_index_lim(double val1,double val2,vector< point > & index);
		int read_index_lim_val(vector<int> xrange,vector<int> yrange,double val1,double val2,vector< point > & index,vector<double> & lval);
		int read_index_lim_val(double val1,double val2,vector< point > & index,vector<double> & lval);
		int store_pix(vector<int> xrange,vector<int> yrange);
		int store_pix();
			
		int index_store_val(double val,vector<point> & index);
		int index_store_val_range(double val,vector<point> & index,int xmin,int xmax,int ymin,int ymax);
		int index_store_lim(double val1,double val2,vector<point> & index);
		
		// Write pixels
		int write_subset(vector<int> xrange,vector<int> yrange,vector<float> value);      // values = argument
		int write_subset(vector<int> xrange,vector<int> yrange);                          // values inside the class
		int write_subset();                          // values inside the class
		int write_px(vector<int> xcoord,vector<int> ycoord,vector<float> val);
		int write_single_px(int xcoord,int ycoord,float val); 
		
		// filter px (replace)
		int store_filter(double vin,double vout);
		int store_filter(double vin,double vout,int xmin,int xmax,int ymin,int ymax);
		int store_filter_tot(double vin,double vout,int xmin,int xmax,int ymin,int ymax);
		
};






#endif /*CFITS_LIB_HPP_*/
