/* History comments (TE):
   ======================

   06.04.2009:
   Changes in the headers which are included to make the code compile
   with g++ 4.3
*/  

#include <iostream> 
#include <fstream> 
#include <sstream>
#include <vector>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>
#include <cmath>
#include "fitsio.h"
#include "CmdLine.h"
#include "fitslib.h"

using namespace std;

int read_val_range (fits infits,double val,vector<int> xrange, vector<int> yrange);
int usage();
int read_lim_range (fits infits,double val1,double val2,vector<int> xrange, vector<int> yrange);
int read_cat(string catfile,vector<double> & lvin,vector<int> & lxmin,vector<int> & lxmax,vector<int> & lymin,vector<int> & lymax);
int read_cat(string catfile,vector<double> & lvin);
int get_pix (fits & infits,vector<double> & lvin,vector<int> & lxmin,vector<int> & lxmax,vector<int> & lymin,vector<int> & lymax);
int get_pix (fits & infits,vector<double> & lvin,int xmin,int xmax,int ymin,int ymax);

int main(int argc, char *argv[]) {

	CCmdLine cmdLine;
	
	// Parse it
	if (cmdLine.SplitLine(argc,argv) < 1) {
		//ASSERT(0);
		exit(-1);	
	}
	
	// Help case
	if (cmdLine.HasSwitch("-h")) {
		usage();
		exit(0);	
	}
	
	// Get the required arguments
	string in;
	try {
		in = cmdLine.GetArgument("-in",0);
	} 
	catch(...) {
		//ASSERT(0);
		cout << "Error ... aruments missing ... \n";
		usage();
		exit(-1);	
	}
	
	// Test if empty arguments
	if (in.size()==0) {
		usage();
		exit(-1);
	}
	
	//	cout << "Infile : " << in << "\n";
	
		// open infits
	fits infits(in.c_str());
	std::ostringstream s,s1;
	string naxis1,naxis2;
	s << infits.get_naxis1();
	naxis1 = s.str();
	s1 << infits.get_naxis2();
	naxis2 = s1.str();
	
	
	// optional arguments
	bool rangeSwitch = cmdLine.HasSwitch("-range");
	
	int xmin = atoi(cmdLine.GetSafeArgument("-x",0,"1").c_str());
	int xmax = atoi(cmdLine.GetSafeArgument("-X",0,naxis1.c_str()).c_str());
	int ymin = atoi(cmdLine.GetSafeArgument("-y",0,"1").c_str());
	int ymax = atoi(cmdLine.GetSafeArgument("-Y",0,naxis2.c_str()).c_str());
	//cout << "ranges " << xmin << " " << xmax << " " << ymin << " " << ymax << "\n";
	
	string cat = cmdLine.GetSafeArgument("-cat",0,"");
	double vmin = atof(cmdLine.GetSafeArgument("-vmin",0,"-1e60").c_str());
	double vmax = atof(cmdLine.GetSafeArgument("-vmax",0,"+1e60").c_str());
	double v = atof(cmdLine.GetSafeArgument("-v",0,"-1e60").c_str());
	
	//cout << "vmin " << vmin << " vmax " << vmax << "\n";  

	// One value 
	if (v != -1e60) {
		vector<int> xrange,yrange;
		xrange.push_back(xmin);
		xrange.push_back(xmax);
		yrange.push_back(ymin);
		yrange.push_back(ymax);
		read_val_range(infits,v,xrange,yrange);

	// One limit	
	} else if (vmin != -1e60 || vmax != 1e60) {
		vector<int> xrange,yrange;
		xrange.push_back(xmin);
		xrange.push_back(xmax);
		yrange.push_back(ymin);
		yrange.push_back(ymax);
		read_lim_range(infits,vmin,vmax,xrange,yrange);

	// One catalog
	} else if (cat != "") {
		vector<double> lvin;
		vector<int> lxmin,lxmax,lymin,lymax;
		if (rangeSwitch) {
			read_cat(cat,lvin,lxmin,lxmax,lymin,lymax);
			get_pix(infits,lvin,lxmin,lxmax,lymin,lymax);
		} else {
			read_cat(cat,lvin);
			get_pix(infits,lvin,xmin,xmax,ymin,ymax);	
		}
	} else {
		usage();
		exit(-1);	
	}
	

	return 0;	
}

int get_pix (fits & infits,vector<double> & lvin,vector<int> & lxmin,vector<int> & lxmax,vector<int> & lymin,vector<int> & lymax) {
	infits.store_pix();
	//vector<point> listP;
	
	int i;
	for(i=0;i<(int)lvin.size();i++) {
		vector<point> listP;
		infits.index_store_val_range(lvin[i],listP,lxmin[i],lxmax[i],lymin[i],lymax[i]);
		// Print the pixels
		vector<point>::iterator it;
		for (it=listP.begin();it!=listP.end();it++) {
			cout << "[" << it->x << "," << it->y << "] = " << lvin[i] << "\n";	
		}
	}

	return 0;
}
int get_pix (fits & infits,vector<double> & lvin,int xmin,int xmax,int ymin,int ymax) {
	infits.store_pix();
	//vector<point> listP;
	
	int i;
	for(i=0;i<(int)lvin.size();i++) {
		vector<point> listP;
		infits.index_store_val_range(lvin[i],listP,xmin,xmax,ymin,ymax);
		// Print the pixels
		vector<point>::iterator it;
		for (it=listP.begin();it!=listP.end();it++) {
			cout << "[" << it->x << "," << it->y << "] = " << lvin[i] << "\n";	
		}
	}

	return 0;	
}


int read_cat(string catfile,vector<double> & lvin,vector<int> & lxmin,vector<int> & lxmax,vector<int> & lymin,vector<int> & lymax) {
	
	// Read the file
	ifstream file;
	file.open(catfile.c_str());
	vector<string> keys;	
	double vin;
	int xmin,xmax,ymin,ymax;
	
	while (! file.eof() ) {
		vin = -1e60;
		file >> vin >> xmin >> xmax >> ymin >> ymax;	
		if (vin != -1e60) {
			lvin.push_back(vin);
			lxmin.push_back(xmin);
			lxmax.push_back(xmax);
			lymin.push_back(ymin);
			lymax.push_back(ymax);
		}
	}
	file.close();

	return 0;	
}

int read_cat(string catfile,vector<double> & lvin) {

	
	// Read the file
	ifstream file;
	file.open(catfile.c_str());
	vector<string> keys;	
	double vin;
	
	while (! file.eof() ) {
		vin = -1e60;
		file >> vin;
		if (vin != -1e60) {
			lvin.push_back(vin);
			//cout << "incat " << vin << " " << vout << "\n";
		}
	}
	file.close();

	return 0;	
}


int read_val_range (fits infits,double val,vector<int> xrange, vector<int> yrange) {

  //cout << "test read " << val << "\n";
	vector<point> listP;
	infits.read_index_val(xrange,yrange,val,listP);
	//cout << "test read 2\n";

	// Print it
	vector<point>::iterator it;
	for (it=listP.begin();it!=listP.end();it++) {
		cout << "[" << it->x << "," << it->y << "] = " << val << "\n";	
	}
	return 0;
}
int read_lim_range (fits infits,double val1,double val2,vector<int> xrange, vector<int> yrange) {
	vector<point> listP;
	vector<double> lval;
	//infits.read_index_lim(xrange,yrange,val1,val2,listP);
	infits.read_index_lim_val(xrange,yrange,val1,val2,listP,lval);
	
	// Print it
	int i = 0;
	vector<point>::iterator it;
	for (it=listP.begin();it!=listP.end();it++) {
		cout << "[" << it->x << "," << it->y << "] = " << lval[i] << " from " << val1 << " " << val2 << "\n";	
		i++;
	}
	return 0;
}
//int read_lim_range (fits infits,double val1,double val2,vector<int> xrange, vector<int> yrange) {
//	vector<point> listP;
//	infits.read_index_lim(xrange,yrange,val1,val2,listP);
//	
//	// Print it
//	vector<point>::iterator it;
//	for (it=listP.begin();it!=listP.end();it++) {
//		cout << "[" << it->x << "," << it->y << "] = " << val1 << " " << val2 << "\n";	
//	}
//	return 0;
//}
int usage() {
	cout << "Usage : getpix file.fits [-vmin VMIN] [-vmax VMAX] [-v VAL] [-cat CAT] [-range] [-x xmin -X xmax -y ymin -Y ymax] \n";	
	return 0;
}
