#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "fitsio.h"
#include "rdf_fits.hpp"
#include<vector>
using namespace std;


int main (int argc, char *argv[]) {

	float **rdf_fits(char * filename,int * nx,int * ny,float * xmin,float * xmax,float * ymin,float * ymax,int verbose);
	fitsfile *fptr;
	float *data,*hough_image,*sins,*coss,*rsins,*rcoss;
	double drho,theta,dtheta,x,y;
	int i,j,k,l,nrho,ntheta,rntheta;
	int status,nfound,nxtmp,nytmp;
    long naxes[2];  
	int Nx = 100;
	int Ny = 100;
	int itmp;
	float refx,refy;
	char image[80],out[80];
    float **temp,xmin,xmax,ymin,ymax;
	float pi = 3.141592654;
	float threshold = 0.5;
	float cell = 0.5;
	char verbose[80];
	int ver = 0;

	itmp = 0;

	// Read the arguments	 
	if (argc < 4) {
		printf("usage : hough input_image output_image threshold_input ... \n");
		exit(1);
	}
	if (argc < 6) {
		refx = 0;
		refy = 0;
	} else {
		refx = atof(argv[4]);
		refy = atof(argv[5]);
	}
	
  	printf("Welcome to hough \n");
  	sscanf(argv[1],"%s",image);
  	sscanf(argv[2],"%s",out);
  	printf ("   input image             : %s \n",image);
  	printf ("   Output image            : %s \n",out);
	
	threshold = atof(argv[3]);
		
	// Verbose level
	if (argc == 7) {
		printf("verbose \n");
		sscanf(argv[6],"%s",verbose);
		if (strcmp(verbose,"-v\n")) {
			ver = 1;	
		}
		printf("verbose ? %s %i \n",verbose,ver);
	}
	
	// Get input image parameters
	status=0;
  	if (fits_open_file(&fptr, image , READONLY, &status)) {
	   printerror( status );           /* call printerror if error occurs */
  	}

 	/* read the NAXIS1 and NAXIS2 keyword to get image size */
  	if ( fits_read_keys_lng(fptr, "NAXIS", 1, 2, naxes, &nfound, &status) ) {
    	   printerror( status );          
  	}
  	if ( fits_close_file(fptr, &status) )      {           /* close the file */
       printerror( status );           
  	}
	
	printf("Size %ld %ld \n",naxes[0],naxes[1]);
    Nx = naxes[0];
    Ny = naxes[1];
    data = (float *) malloc(Nx*Ny*sizeof(float));
	
	// Read the input image
	temp=(float **)rdf_fits(image,&nxtmp,&nytmp,&xmin,&xmax,&ymin,&ymax,itmp);
	
	/* copy the data into the data array + Build the mask */
    for (i=0,j=0; j<Ny; j++) {
		for (k=0; k<Nx; k++,i++) {
	  
	  	// Copy data
	  		data[i]= (float) temp[j][k];
	  		//if (data[i] > 1.0) {
	  		//	printf("val %i %i %f \n",j,k,data[i]);	
	  		//}
	  		
	  		//if (j==4 && k==49) {
	  		//	printf("val 49 3 %f\n",data[i]);	
	  		//}
		}
    }
    
	  
	
	// Create the hough image
	drho = 1.0/sqrt(2.0);
	nrho = (int) (sqrt(Nx*Nx+Ny*Ny)/drho) ;
	nrho++;
	ntheta = 1800;
	rntheta = 2*ntheta;
	dtheta = pi/ntheta;
	
	
	hough_image = (float *) malloc(2*nrho*ntheta*sizeof(float));
	for(i=0;i<2*nrho*ntheta;i++) {
		hough_image[i] = 0.0;	
	}
	
	
	// Precompute sin and cos angles
	printf("PreCalculate sin/cos ...\n");
	sins = (float *) malloc(ntheta*sizeof(float));
	coss = (float *) malloc(ntheta*sizeof(float));
	rsins = (float *) malloc(2*ntheta*sizeof(float));
	rcoss = (float *) malloc(2*ntheta*sizeof(float));
	
	for(i=0;i<ntheta;i++) {
		theta = ((double)i+0.5)*dtheta;	
		sins[i] = sin(theta);
		coss[i] = cos(theta);
	}
	for(i=0;i<rntheta;i++) {
		theta = ((double)i-ntheta+0.5)*dtheta;	
		if (i>=ntheta) {
			rsins[i] = sins[i-ntheta];
			rcoss[i] = coss[i-ntheta];
		} else {
			rsins[i] = -sins[(ntheta-i)-1];
			rcoss[i] = -coss[(ntheta-i)-1];
		}
	}	
	
	
	// Calculate the hough transform
	printf("Calculate the hough transform ...\n");
	for(j=0;j<Ny;j++) {
		y = (double) j - refy;
		if (ver == 1){
			printf("line %f of %i \n",y,Ny);
		}
		for(i=0;i<Nx;i++) {
			if(data[j*Nx+i]>threshold) {
				//printf("point %i of %i : %f to %f \n",i,j,data[j*Nx+i],threshold);
				x = (double) i -refx;
				for(k=0;k<ntheta;k++) {
					l = (int)(floor((x * coss[k] + y * sins[k])/drho));
					if (l < 0) {
						l *= -1;
						hough_image[k*nrho+l] += 1.0; 	 
					} else {
						hough_image[(k+ntheta)*nrho+l] += 1.0;  
					}
				}
			}
		}
	}
	
	
	write_fits (hough_image, nrho,rntheta,cell,out);
	
	// Write some keywords
	vector<string> keys,coms;
	vector<double> vals,vals2;
	keys.push_back("REFX");
	vals.push_back(refx);
	keys.push_back("REFY");
	vals.push_back(refy);
	keys.push_back("DRHO");
	vals.push_back(drho);
	vals2.push_back(-drho);
	keys.push_back("DTHETA");
	vals.push_back(dtheta);
	vals2.push_back(dtheta);
	coms.push_back("ref px (X)");
        coms.push_back("ref px (Y)");
        
	coms.push_back("scale in rho");
	coms.push_back("scale in theta");
	write_key_num(out,keys,vals,coms);
	
	// Free temp
    printf("Free temp \n"); 
   	for (j=0; j<Ny; j++) {
		free(temp[j]);
    }
    free(temp);
    free(data);
	free(hough_image);
	free(sins);
	free(coss);
	free(rsins);
	free(rcoss);
	//free(flag);
	return 0;
}



