#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "rdf_fits.hpp"

/*--------------------------------------------------------------------------*/
int printerror( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/

    if (status)
    {
       fits_report_error(stderr, status); /* print error report */

       exit( status );    /* terminate the program, returning error status */
    }
    return 0;
}
/*--------------------------------------------------------------------------*/

float	**alloc_square_float(int nbr_lin,int nbr_col) {
auto     float   **square;
register int      i, j;

square = (float **) malloc((unsigned) nbr_lin*sizeof(float *));
if (square != 0)
  {
  for (i=0; i<nbr_lin; i++)
    {
    square[i] = (float *)malloc((unsigned) nbr_col*sizeof(float));
    if (square[i] == 0) square = 0;
    }
  }

for (i=0; i<nbr_lin; i++)
for (j=0; j<nbr_col; j++)
square[i][j]=0.0;

return(square);
}

int free_square(float ** resf,int ny,int nx) {
	int i;
	for (i=0; i<ny; i++) {
		free(resf[i]);	
	}
	free(resf);
		
	return 0;
}

int write_fits_tab (float ** tab, int nx, int ny, char * filename) {
	int i,j;
	int count = -1;
	float * image;
	image = (float *) malloc(nx * ny * sizeof(float));
	for(i=0;i<ny;i++) {
		for(j=0;j<nx;j++) {
			count++;
			image[count] = tab[i][j];
		}	
	}
	write_fits (image,nx,ny,0,filename);
	free(image);
	
	return 0;
}

float ** rdf_fits(char * filename,int * nx,int * ny,float * xmin,float * xmax,float * ymin,float * ymax,int verbose) {
    fitsfile *fptr;       // pointer to the FITS file, defined in fitsio.h 
    int status, ii, jj, kk;
    int nfound, anynull;
    long nnaxes[2];   
    long fpixel, nbuffer, npixels;
    int nkeys, keypos;
    char card[FLEN_CARD];

    float crval1 = 0.0,crpix1 = 0.0,cdelt1 = 0.0;
    float crval2 = 0.0,crpix2 = 0.0,cdelt2 = 0.0;
    float **image;
    float **alloc_square_float(int ny,int nx);
#define buffsize 1000
    float datamin, datamax, nullval, buffer[buffsize];
    float fval;

    //printf("Test \n");

    status = 0;         // initialize status before calling fitsio routines 

    if (fits_open_file(&fptr, filename, READONLY, &status)) 
         printerror( status );           // call printerror if error occurs 

    // read the NAXIS1 and NAXIS2 keyword to get image size 
    if ( fits_read_keys_lng(fptr, "NAXIS", 1, 2, nnaxes, &nfound, &status) )
         printerror( status );          

    //printf("Test1 \n");

  fpixel = 1;                               // first pixel in the image    
  npixels = nnaxes[0] * nnaxes[1];          // number of pixels in the image 
  *nx = (int) nnaxes[0];
  *ny = (int) nnaxes[1];

  //printf("Test \n");

  //printf("Max int %d %d \n",*nx,*ny);

  // image = (float **) alloc_square_float(int *ny,int *nx);
  image = alloc_square_float(*ny,*nx);
  ii=jj=0;

  //printf("Test \n");

    nullval  = 0;                // don't check for null values in the image 
    datamin  = 1.0E30;
    datamax  = -1.0E30;

    while (npixels > 0)
    {
      nbuffer = npixels;
      if (npixels > buffsize)
        nbuffer = buffsize;     // read as many pixels as will fit in buffer 
      
      // Note that even though the FITS images contains unsigned integer 
      // pixel values (or more accurately, signed integer pixels with    
      // a bias of 32768),  this routine is reading the values into a    
      // float array.   Cfitsio automatically performs the datatype      
      // conversion in cases like this.                                  
      
      if ( fits_read_img(fptr, TFLOAT, fpixel, nbuffer, &nullval,
			 buffer, &anynull, &status) )
	printerror( status );
      
      //printf("Test1 \n");

      for (kk = 0; kk < nbuffer; kk++)  
	{
	  // printf("Test1 %d %d %d %d \n",kk,nbuffer,jj,ii);

	  image[jj][ii++]=buffer[kk];
	  if (ii>= *nx)
	    { jj++; ii=0; }
	  
	  if ( buffer[kk] < datamin )
            datamin = buffer[kk];
	  
	  if ( buffer[kk] > datamax )
            datamax = buffer[kk];
	}
      //printf("Test2 \n");
      npixels -= nbuffer;    // increment remaining number of pixels 
      fpixel  += nbuffer;    // next pixel to be read in image 
    }
    
    if (verbose==1) {
      printf("\nMin and max image pixels =  %.0f, %.0f\n", datamin, datamax);
    }

    

  // read keywords 
        if (fits_get_hdrpos(fptr, &nkeys, &keypos, &status) )
            printerror( status );

        for (kk = 1; kk <= nkeys; kk++)  {
            if ( fits_read_record(fptr, kk, card, &status) )
                 printerror( status );

	    if (verbose==1) {
	      printf("%s\n", card); // print the keyword card 
	    }

    if (strncmp(card,"CRVAL1",6)==0)
        {
        sscanf(card, "CRVAL1  = %f", &fval);
        crval1=fval;
        }
    else if (strncmp(card,"CRPIX1",6)==0)
        {
        sscanf(card, "CRPIX1  = %f", &fval);
        crpix1=fval;
        }
    else if (strncmp(card,"CDELT1",6)==0)
        {
        sscanf(card, "CDELT1  = %f", &fval);
        cdelt1=fval;
        }
    else if (strncmp(card,"CRVAL2",6)==0)
        {
        sscanf(card, "CRVAL2  = %f", &fval);
        crval2=fval;
        }
    else if (strncmp(card,"CRPIX2",6)==0)
        {
        sscanf(card, "CRPIX2  = %f", &fval);
        crpix2=fval;
        }
    else if (strncmp(card,"CDELT2",6)==0)
        {
        sscanf(card, "CDELT2  = %f", &fval);
        cdelt2=fval;
        }

        }

    if ( fits_close_file(fptr, &status) )               // close the file 
         printerror( status );           

    if (verbose==1) {
  printf("%.2f %.2f %.2f %.2f %.2f %.2f\n",crval1,cdelt1,crpix1,
					crval2,cdelt2,crpix2);
    }
  *xmin=crval2 + (crpix2 -1)*(cdelt2);
  *xmax=crval2 + (*nx-crpix2)*(cdelt2);
  *ymin=crval1 + (crpix1 -1)*(cdelt1);
  *ymax=crval1 + (*ny-crpix1)*(cdelt1);

  if (verbose==1) {
    printf("%.2f %.2f %.2f %.2f \n", *xmin, *xmax, *ymin, *ymax);
  }
  //free(fptr);
  
  return ((float **) image);
}




// Simples routines to wrte a fits file with CFITSIO



//--------------------------------------------------------------------------
int delete_file (char * filename, int status) {
  //int unit,blocksize;
  fitsfile *fptr;

  if (status > 0) {
    return status;
  } 
  
  // Try to open the file
  if (!fits_open_file(&fptr, filename, READONLY, &status)) {
    // file opened / delete it
    ffdelt(fptr,&status);
  } else if (status == 103) {
    // file doesn't exist
    status = 0;
    ffcmsg ();
  } else {
    // other error / delete the file
    status = 0;
    ffcmsg ();
    ffdelt(fptr,&status);
  }
  
  // free the pointer
  // free(fitsfile);
	return 0;
} 
//--------------------------------------------------------------------------
int write_key_num (char * file,vector<string> & keys,vector<double> & vals, vector<string> & coms) {
	
	fitsfile *fptr;
	int status=0; int ii = -1;
	ffopen(&fptr ,file,1, &status);
	if (status > 0) {
    	printerror(status);
    	return status;
  	}
	char key[80];
	char com[80];
	
	vector<string>::iterator it;
	ii = -1;
	for(it=keys.begin();it!=keys.end();it++) {
		ii++;
		int decimals = 10;
		
		sscanf(keys[ii].c_str(),"%s",key);
		sscanf(coms[ii].c_str(),"%s",com);
		float tmp = (float) vals[ii];
		ffpkyf(fptr,key,tmp,decimals,com,&status);
	}
	ffclos(fptr,&status);

  	if (status > 0) {
    	printerror(status);
  	}
  	return status;
}
// --------------------------------------------------------------------------
int write_fits (float * image,int nx , int ny,float cell,char * filename) {
  
  //int unit;
  int status,blocksize,group,fpixel,nelements;
  int bitpix,naxis,xrefpix,yrefpix,decimals;
  long naxes[2];
  float ra,dec,z,skycell;
  int simple,extend;
  fitsfile *fptr;
  
  int delete_file (char * filename, int status);
  int printerror( int status);

  char key[80];
  char val[80];
  char com[80];
  status = 0;
  blocksize = 1;

  // Create the output file
  delete_file(filename,status);
  ffinit(&fptr ,filename, &status);

  // Initialize variables
  group = 1;
  fpixel=1;
  nelements=nx*ny;
  simple=1;
  bitpix=-32;
  naxis=2;
  naxes[0]=nx;
  naxes[1]=ny;
  extend=1;

  ra = 0.0;
  dec = 0.0;
  skycell = cell/3600.0;
  xrefpix = nx/2+1;
  yrefpix = ny/2+1;
  z = 0.0;
  decimals = 10;

  // Write compulsory part of header:
  ffphpr(fptr,simple,bitpix,naxis,naxes,0,1,extend,&status);


  // optionnal keywords
  sscanf("CTYPE1","%s",key);
  sscanf("RA---SIN","%s",val);
  sscanf("Axis1","%s",com);
  ffpkys(fptr, key, val,com,&status);
  sscanf("CTYPE2","%s",key);
  sscanf("DEC--SIN","%s",val);
  sscanf("Axis2","%s",com);
  ffpkys(fptr, key, val,com,&status);
  
  sscanf("CRVAL1","%s",key);
  sscanf("Coordinate 1 at ref pixel","%s",com);
  ffpkyf(fptr,key,ra,decimals,com,&status);
  sscanf("CRVAL2","%s",key);
  sscanf("Coordinate 2 at ref pixel","%s",com);
  ffpkyf(fptr,key,dec,decimals,com,&status);
  
  sscanf("CDELT1","%s",key);
  sscanf("Coordinate 1/pixel","%s",com);
  ffpkyf(fptr,key,skycell,decimals,com,&status);
  sscanf("CDELT2","%s",key);
  sscanf("Coordinate 2/pixel","%s",com);
  ffpkyf(fptr,key,skycell,decimals,com,&status);
  
  sscanf("CRPIX1","%s",key);
  sscanf("Reference pixel 1","%s",com);
  ffpkyj(fptr,key,xrefpix,com,&status);
  sscanf("CRPIX2","%s",key);
  sscanf("Refernce pixel 2","%s",com);
  ffpkyj(fptr,key,yrefpix,com,&status);
  
  sscanf("CROTA1","%s",key);
  sscanf("Rotation 1","%s",com);
  ffpkyf(fptr,key,z,decimals,com,&status);
  sscanf("CROTA2","%s",key);
  sscanf("Rotation 2","%s",com);
  ffpkyf(fptr,key,z,decimals,com,&status);
  
  sscanf("LTV1","%s",key);
  sscanf("Physical to image 1","%s",com);
  ffpkyj(fptr,key,xrefpix,com,&status);
  sscanf("LTV2","%s",key);
  sscanf("Physical to image 2","%s",com);
  ffpkyj(fptr,key,yrefpix,com,&status);
  
  sscanf("LTM1_1","%s",key);
  sscanf("Physical to image","%s",com);
  ffpkyf(fptr,key,1.0/cell,decimals,com,&status);
  sscanf("LTM2_2","%s",key);
  sscanf("Physical to image","%s",com);
  ffpkyf(fptr,key,1.0/cell,decimals,com,&status);

  // Write datas
  printf("image %d %d %d %d \n",nx,ny,naxis,status);
  ffp2de(fptr,group,nx,nx,ny,image,&status);
  printf("image %d %d %d \n",nx,ny,status);

  //Close file
  ffclos(fptr,&status);

  if (status > 0) {
    printerror(status);
  }
  return status;
}
//--------------------------------------------------------------------------






