#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "fitsio.h"
#include "rdf_fits.hpp"

int main (int argc, char *argv[]) {
 
	float **rdf_fits(char * filename,int * nx,int * ny,float * xmin,float * xmax,float * ymin,float * ymax,int verbose);
	fitsfile *fptr;
	float *hough_image;
	double theta;
	float drho,dtheta;
	int i,j,k,l,nrho,ntheta,flagg,nrtheta;
	int status,nfound,nxtmp,nytmp;
    long naxes[2];  
	int Nx = 100;
	int Ny = 100;
	int itmp;
	float edge,rho,a,b;
	char image[80],out[80],himage[80];
    float **temp,xmin,xmax,ymin,ymax;
	float pi = 3.141592654;
	float threshold2 = 260;
	float cell = 0.5;
	float * flag;
	char verbose[80];
	int ver = 0;

	itmp = 0;

	// Read the arguments	 
	if (argc < 5) {
		printf("usage : hough input_hough_image ref_image output_image threshold_hough ... \n");
		exit(1);
	}

  	printf("Welcome to hough \n");
  	sscanf(argv[1],"%s",himage);
  	sscanf(argv[2],"%s",image);
  	sscanf(argv[3],"%s",out);
  	printf ("   input image             : %s \n",image);
  	printf ("   Output image            : %s \n",out);
	
	threshold2 = atof(argv[4]);
	
	// Verbose level
	if (argc == 6) {
		printf("verbose \n");
		sscanf(argv[5],"%s",verbose);
		if (strcmp(verbose,"-v\n")) {
			ver = 1;	
		}
		printf("verbose ? %s %i \n",verbose,ver);
	}
	
	// ------------------------ READ THE REFERENCE IMAGE ----------------------
	
	// Get input image parameters
	status=0;
  	if (fits_open_file(&fptr, image , READONLY, &status)) { 
	   printerror( status );           /* call printerror if error occurs */
  	}

 	/* read the NAXIS1 and NAXIS2 keyword to get image size */
  	if ( fits_read_keys_lng(fptr, "NAXIS", 1, 2, naxes, &nfound, &status) ) {
    	   printerror( status );          
  	}
  	
  	
  	if ( fits_close_file(fptr, &status) )      {           /* close the file */
       printerror( status );           
  	}
	
	printf("Size %ld %ld \n",naxes[0],naxes[1]);
	
    Nx = naxes[0];
    Ny = naxes[1];
    //hough_image = (float *) malloc(Nx*Ny*sizeof(float));
	
	
	
	// ------------------------ READ THE HOUGH IMAGE ----------------------
	
		// Get input image parameters
	status=0;
  	if (fits_open_file(&fptr, himage , READONLY, &status)) {
	   printerror( status );           /* call printerror if error occurs */
  	}

 	/* read the NAXIS1 and NAXIS2 keyword to get image size */
  	if ( fits_read_keys_lng(fptr, "NAXIS", 1, 2, naxes, &nfound, &status) ) {
    	   printerror( status );          
  	}
  	char comm[80]; drho = 0.0; dtheta = 0.0;
  	if ( ffgky(fptr,TFLOAT, "DRHO", &drho,comm, &status) ) {
    	   printerror( status );          
  	}
  	if ( ffgky(fptr,TFLOAT, "DTHETA", &dtheta,comm, &status) ) {
    	   printerror( status );          
  	}
  	if ( fits_close_file(fptr, &status) )      {           /* close the file */
       printerror( status );           
  	}
	
	//printf("Size %d %d \n",naxes[0],naxes[1]);
	//printf("drho/dtheta %.10f %.10f \n",drho,dtheta);
	
	// Create the hough image
	if (drho == 0) {
		drho = 1.0/sqrt(2.0);
	}
	nrho = (int) (sqrt(Nx*Nx+Ny*Ny)/drho) ;
	nrho++;
	ntheta = 1800;
	nrtheta = 2*ntheta;
	if (dtheta == 0) {
		dtheta = pi/ntheta;
	}
	
    nrho = naxes[0];
    nrtheta = naxes[1];
    ntheta = nrtheta / 2;
    hough_image = (float *) malloc(nrtheta*nrho*sizeof(float));
	
	
	// Read the input image
	temp=(float **)rdf_fits(himage,&nxtmp,&nytmp,&xmin,&xmax,&ymin,&ymax,itmp);
	
	/* copy the data into the data array + Build the mask */
    for (i=0,j=0; j<nrtheta; j++) {
		for (k=0; k<nrho; k++,i++) { 
	  
	  	// Copy data
	  		hough_image[i]= (float) temp[j][k];
		}
    }
    
	  
	
	// ------------------------------- INVERSE HOUGH TRANSFORM --------------------------
	
	printf("Inverse houghT \n");
	
	// Build the flagged map
	flag = (float*) malloc (Nx*Ny*sizeof(float));
	edge = sqrt(2.0)/2.0;
	
	for(j=0;j<Nx;j++) {
		for(i=0;i<Ny;i++) {
			flag[i*Nx+j] = 0.0;
		}
	}
	
	printf("Inverse houghT \n");
	
	flagg = 0;
	for(j=0;j<nrtheta;j++) {
		//theta = ((double)j+0.5)*dtheta;
		theta = (((double)j-ntheta)+0.5)*dtheta;
		for(i=0;i<nrho;i++) {
			rho = (double) i * drho;	
				
			if (hough_image[j*nrho+i] > threshold2) {
				//printf("val j i  %i %i \n",i,j);
				if (abs(sin(theta) > edge)) {
					//printf("case1 \n");
					a = -cos(theta)/sin(theta);
					b = rho / sin(theta);	
					//b = rho / sin(theta) - a * rho * cos(theta);
					for(l=0;l<Nx;l++) {
						k = (int)(floor( (double) l * a + b ));	
						//printf("case1 val  nx ny %i %i \n",l,k);
						
						if ((k>=0) && (k<Ny)) {
							flag[k*Nx+l] = 1.0;
							//printf("val 1 nx ny %i %i \n",l,k);
						}
					}
				} else {
					//printf("case2 \n");
					
					a = -sin(theta)/cos(theta);
					b = rho / cos(theta);	
					for(k=0;k<Ny;k++) {
						l = (int)(floor( (double) k * a + b ));
						//printf("case2 val %f %f  nx ny %i %i \n",a,b,l,k);
							
						if ((l>=0) && (l<Nx)) {
							flag[k*Nx+l] = 1.0;
							//printf("val 1 nx ny %i %i \n",l,k);
						}
					}
				}	
			}	
		}
	}
	
	write_fits (flag, Nx,Ny,cell,out);
	
	// Free temp
    printf("Free temp \n");
   	for (j=0; j<ntheta; j++) {
		free(temp[j]);
    }
    free(temp);
    free(hough_image);
	//free(flag);
	return 0;
}



