#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "fitsio.h"
#include "rdf_fits.hpp"
#include<vector>
using namespace std;

//int get_intersect(float eq1,float eq2,vector<int> limits, float & p1x,float & p1y,float & p2x, float & p2y);
int get_intersect(float eq1,float eq2,vector<int> limits, float p1[2],float  p2[2]);
int get_inter(float eq1[3], float eq2[3], float  pt[2]);
int line_eq(float p1[2], float p2[2], float  eq[3]);

int main (int argc, char *argv[]) {

	int nx,ny,nwx,nwy;
	int i,j,check;
	//float p1x,p1y,p2x,p2y;
	float p1[2],p2[2];
	char output[80];
	float drho,dtheta,weight,eq1,eq2;
	vector<int> limits;
	float * weight_im;
	float cell = 0.5;
	int refx = 0, refy = 0;
	
	// Read the arguments		 
	if (argc < 4) {
		printf("usage : hough_inter_ref NAXIS1 NAXIS2 output refx refy [drho dtheta] \n");
		exit(1);
	}

  	printf("Welcome to hough \n");
  	nx = atoi(argv[1]);
	ny = atoi(argv[2]);
  	sscanf(argv[3],"%s",output);
  	printf ("   naxis1/naxis2           : %d,%d \n",nx,ny);
  	printf ("   Output image            : %s \n",output);
	
	if (argc < 4) {
		printf("usage : hough_inter_ref NAXIS1 NAXIS2 output refx refy [drho dtheta] \n");
		exit(1);
	}
	if (argc == 6) {
		refx = atoi(argv[4]);
		refy = atoi(argv[5]);
		drho = 1.0/sqrt(2);
		dtheta = 3.141592654/1800; 	
	}
	if (argc == 8) {
		refx = atoi(argv[4]);
		refy = atoi(argv[5]);
		drho = atof(argv[6]);
		dtheta = atof(argv[7]); 	
	}
	
	// Get the size of the hough weight image
	nwx = int(sqrt(nx*nx+ny*ny)/drho+1);
	nwy = int (3.141592654 / dtheta * 2);
	
	// limits of the chip
	limits.push_back(0-refx);
	limits.push_back(nx-refx);
	limits.push_back(0-refy);
	limits.push_back(ny-refy);
	
	// Loop on the hough weight chip image
	weight_im = (float *) malloc (nwx*nwy*sizeof(float));
	
	for (i=0;i<nwx;i++) {
		//printf("line %d of %d \n",i,nwx);
		for (j=0;j<nwy;j++) {
			
			// edges 
			if (i < 2 || j < 2 || i > nwx -2 || j > nwy -2) {
				weight = -1;	
			} else {
				
				// line equation
				eq1 = i*drho;
				eq2 = ((j-nwy/2)+0.5)*dtheta;
		
				//eq1 = 100;
				//eq2 = -3.1415927/3*2;
		
				// Get intersection points
				//printf("eq %f %f \n",eq1,eq2);
				check = get_intersect(eq1,eq2,limits,p1,p2);
				//printf("check %d and %f %f %f %f \n",check,p1[0],p1[1],p2[0],p2[1]);
				
				if (check != 0) {
					weight = 0;
				} else {
				
					weight = p1[0]+p2[0]/10.0;
				}
			}
			
			// Put in list
			//printf("weight = %f \n",weight);
			weight_im[j*nwx+i] = weight;
			
			
		}
	}

	// write it
	write_fits(weight_im,nwx,nwy,cell,output);

	// write some keywords
	vector<string> keys,coms;
	vector<double> vals;
	keys.push_back("DRHO");
	vals.push_back(drho);
	keys.push_back("DTHETA");
	vals.push_back(dtheta);
	coms.push_back("scale in rho");
	coms.push_back("scale in theta");
	write_key_num(output,keys,vals,coms);

	free(weight_im);
}

//int get_intersect(float eq1,float eq2,vector<int> limits, float & P1x,float & P1y,float & P2x, float & P2y) {
int get_intersect(float eq1,float eq2,vector<int> limits, float P1[2],float P2[2]) {
	
	
	float p1[2],p2[2],eq[3],eq_ref[3];
	float pt[2];
	int check;
	
	P1[0] = -1.0;
	P2[0] = -1.0;
	
	// Get a cost + b sint = r equation
	eq_ref[0] = cos(eq2);
	eq_ref[1] = sin(eq2);
	eq_ref[2] = eq1;
	
	//printf("eqref %f %f %f \n",eq_ref[0],eq_ref[1],eq_ref[2]);
	
	// side 1
	p1[0] = limits[0];
	p1[1] = limits[2];
	p2[0] = limits[1];
	p2[1] = limits[2];
	line_eq(p1,p2,eq);
	//printf("lines1 p1 %f %f p2 %f %f ... eq %f %f %f \n",p1[0],p1[1],p2[0],p2[1],eq[0],eq[1],eq[2]);
	check = get_inter(eq,eq_ref,pt);
	//printf("inter %d %f %f \n",check,pt[0],pt[1]);
	if (check == 0 && pt[0] >= p1[0] && pt[0] <= p2[0]) {
		if (P1[0] == -1) {
			P1[0] = 1;
			P1[1] = pt[1];
		} else if (P2[0] == -1) {
			P2[0] = 1;
			P2[1] = pt[1];				
		} else {
			return -1;	
		}
	}
	
	// side 2
	p2[0] = limits[1];
	p2[1] = limits[3];
	p1[0] = limits[1];
	p1[1] = limits[2];
	line_eq(p1,p2,eq);
	//printf("lines2 p1 %f %f p2 %f %f ... eq %f %f %f \n",p1[0],p1[1],p2[0],p2[1],eq[0],eq[1],eq[2]);
	check = get_inter(eq,eq_ref,pt);
	//printf("inter %d %f %f \n",check,pt[0],pt[1]);
	if (check == 0 && pt[1] >= p1[1] && pt[1] <= p2[1]) {
		if (P1[0] == -1) {
			P1[0] = 2;
			P1[1] = pt[1];
		} else if (P2[0] == -1) {
			P2[0] = 2;
			P2[1] = pt[1];				
		} else {
			return -1;	
		}
	}
	
	// side 3
	p1[0] = limits[0];
	p1[1] = limits[3];
	p2[0] = limits[1];
	p2[1] = limits[3];
	line_eq(p1,p2,eq);
	//printf("lines3 p1 %f %f p2 %f %f ... eq %f %f %f \n",p1[0],p1[1],p2[0],p2[1],eq[0],eq[1],eq[2]);
	check = get_inter(eq,eq_ref,pt);
	//printf("inter %d %f %f \n",check,pt[0],pt[1]);
	if (check == 0 && pt[0] >= p1[0] && pt[0] <= p2[0]) {
		if (P1[0] == -1) {
			P1[0] = 3;
			P1[1] = pt[1];
		} else if (P2[0] == -1) {
			P2[0] = 3;
			P2[1] = pt[1];				
		} else {
			return -1;	
		}
	}
	
	// side 4
	p1[0] = limits[0];
	p1[1] = limits[2];
	p2[0] = limits[0];
	p2[1] = limits[3];
	line_eq(p1,p2,eq);
	//printf("lines4 p1 %f %f p2 %f %f ... eq %f %f %f \n",p1[0],p1[1],p2[0],p2[1],eq[0],eq[1],eq[2]);
	check = get_inter(eq,eq_ref,pt);
	//printf("inter %d %f %f \n",check,pt[0],pt[1]);
	if (check == 0 && pt[1] >= p1[1] && pt[1] <= p2[1]) {
		if (P1[0] == -1) {
			P1[0] = 4;
			P1[1] = pt[1];
		} else if (P2[0] == -1) {
			P2[0] = 4;
			P2[1] = pt[1];				
		} else {
			return -1;	
		}
	}
	
	
	if (P1[0] != -1.0 && P2[0] != -1.0) {
		return 0;
	} else {
		return -1;	
	}
	
}

int line_eq(float p1[2], float p2[2], float eq[3]) {
	if (p1[0] != p2[0]) {
		eq[0] = -(p2[1]-p1[1])/(p2[0]-p1[0]);
		eq[1] = 1;
		eq[2] = p2[1] + eq[0] * p2[0];
	} else {
		eq[0] = 1;
		eq[1] = 0;
		eq[2] = p2[0];
	}
	
	return 0;
}

int get_inter(float eq1[3], float eq2[3], float pt[2]) {
	float a,b,c,d,det;
	a = eq1[0];
	b = eq1[1];
	c = eq2[0];
	d = eq2[1];
	
	det = a * d - b * c;
	if (det == 0) {
		return -1;	
	} else {
		pt[0] = (d*eq1[2]-b*eq2[2])/det;
		pt[1] = (a * eq2[2] - c * eq1[2])/det;	
		return 0;
	}
}
