#!/bin/bash -x

# the script creates hough images for science frames.
# 
# First version : 31/01/2007
# 
# 06.04.2009:
# cleaning up of script (TE)

# $1: main directory
# $2: science dir.
# $3: image extension (ext) on ..._iext.fits (i is the chip number)
# $4: chips to be processed

. ${INSTRUMENT:?}.ini

# parameters 
ELL=0.3
WEIGHTDIR="/$1/WEIGHTS"
TEMPDIR=.

# export PWD so that progs.ini can be read
# from a Perl script:
PROGS_DIR=${PWD}
export PROGS_DIR;
export PIPESOFT;

CHIP=$4
ls -1 /$1/$2/*_${CHIP}$3.fits > ${TEMPDIR}/hough_images_$$

DIR=`pwd`

# all temporary Hough images are created in the
# science directory:
cd /$1/$2

# The following while loop is inefficient. The looping
# should be done within the perl script!
while read file
do
  # Create the segmentation image
  BASE=`basename ${file} $3.fits`
  SEG=${BASE}$3_SEG.fits
  WEIGHT=${BASE}$3_SEG_hSN.fits

  if [ ! -f ${WEIGHT} ]; then
    perl ${S_STRACK_HOUGH} \
         -seg ${SEG} \
         -instrum ${INSTRUMENT} \
         -hweight_only \
         -hweight_fin HTweight_${INSTRUMENT}.fits
  fi
done < ${TEMPDIR}/hough_images_$$

# clean up and bye:
test -f ${TEMPDIR}/hough_images_$$ && rm  ${TEMPDIR}/hough_images_$$
