#!/bin/bash -u

# script to create an instrument configuration file for the
# stracktools. This script creates the Stract_${INSTRUMENT}.ini file.
# The script needs to be run withn a THELI reduce directory.

#
# Besides running this script you manually need to update the strack
# configuration files instrum_conf.ini and
# ${INSTRUMENT}_layout.ini. Have a look at example files to get the
# necessary syntax. As REFX and REFY within instrum_conf.ini take the
# CRPIX1 and CRPIX2 values from the co-added image that is also used
# for this script!

# preconditions:
# - a THELI configuration file for the instrument.
# - a co-added image from the new instrument with corresponding
#   scamp headers from (one) image entering the co-added data.
# - The script needs the programs sky2xy in the PATH.

# The script implicitely assumes that all chips of the instrument
# have the same size. Releveant is the size after trimming overscan etc.
# sections.

#$1: full path to the coadded image
#$2: full path to the scamp headers directory
#$3: image name from the single image that is used to
#    create the config file; it does not matter which
#    image you give as long as the scamp information is fine.
#    scamp headers have names as $3_CHIP.head.

. ./${INSTRUMENT:?}.ini

IMAGE=$1
HEADERDIR=$2
IMANAME=$3

# we finally need pixel coordinates from the bottom left and top right
# corner of all the single-frame chips within the co-added image.

# first get a dummy LDAC catalogue with the lower-left and upper-right
# pixel coordinates from the chips (those are assumed to be the same for
# all chips!)
echo "1 1"                     > pos.txt_$$
echo "${SIZEX[1]} 1"           >> pos.txt_$$
echo "1 ${SIZEY[1]}"           >> pos.txt_$$
echo "${SIZEX[1]} ${SIZEY[1]}" >> pos.txt_$$

{
  echo "COL_NAME  = X_IMAGE"
  echo "COL_TTYPE = FLOAT"
  echo "COL_HTYPE = FLOAT"
  echo "COL_UNIT  = pix"
  echo "COL_COMM  = x_position"
  echo "COL_DEPTH = 1"
  echo "#"
  echo "COL_NAME  = Y_IMAGE"
  echo "COL_TTYPE = FLOAT"
  echo "COL_HTYPE = FLOAT"
  echo "COL_UNIT  = pix"
  echo "COL_COMM  = y_position"
  echo "COL_DEPTH = 1"
} >> asctoldac.conf_$$

${P_ASCTOLDAC} -i pos.txt_$$ -o chip.ldac_$$ -c asctoldac.conf_$$

test -f aplastrom.txt_$$ && rm aplastrom.txt_$$
for i in `seq 1 ${NCHIPS}`
do
  echo "chip.ldac_$$ ${HEADERDIR}/${IMANAME}_${i}.head chip_${i}.ldac_$$" >> \
    aplastrom.txt_$$
done

./aplastrom_scamp.py -t OBJECTS -f aplastrom.txt_$$

{
  echo "# 1 PERIOD"
  echo "# 2 CHIP"
  echo "# 3 X_1_1"
  echo "# 4 Y_1_1"
  echo "# 5 X_X_1"
  echo "# 6 Y_X_1"
  echo "# 7 X_1_Y"
  echo "# 8 Y_1_Y"
  echo "# 9 X_X_Y"
  echo "# 10 Y_X_Y"
} > Strack_${INSTRUMENT}.ini

for i in `seq 1 ${NCHIPS}`
do
  ${P_LDACTOASC} -b -i chip_${i}.ldac_$$ -t OBJECTS \
                 -k ALPHA_J2000_corr DELTA_J2000_corr | \
  while read RA DEC
  do
    sky2xy ${IMAGE} ${RA} ${DEC}
  done | \
  ${P_GAWK} 'BEGIN {printf("%3d", '${i}')} {
    printf("%8d %8d", $5 + 0.5, $6 + 0.5)} END {printf("\n")}' >> \
    Strack_${INSTRUMENT}.ini
done

# and bye:
rm *_$$

exit 0;
