#!/usr/bin/perl -w

############################################################################
# Build the hough transform image
# input : seg     : segmentation image
#       : hweight : housgh weight image 
#  	: out     : output root name 
#       : bin     : binning factor
#       : refx    : ref px for hough transform (def = 0)
#       : refy    : ref px for hough transform (def = 0)
#
# WEIGHT TUNING
#       : hwmin   : hough weight minimum clipping
#       : hwmax   : hough weight maximum clipping (def = max)
#       : hwcheck : min value for mosaic check (def = hwmin)
#       : hwtcheck: theta limit for vertical line mosaic check
#       : hwscale : scaling for the weight images
#
# POST HOUGH
#       : hclip   : clip at value hclip and print fits files
############################################################################

# HISTORY COMMENTS (TE):
#
# 10.01.2013:
# I started the clean up the code a bit.

if(defined $ENV{'PERL_LIB'}) {
    push(@INC,$ENV{'PERL_LIB'});
}

$homedir = $ENV{'HOME'};
push(@INC,"$homedir/perl/VIRMOS/pipeline");
push(@INC,$ENV{"PIPESOFT"}."/stracktools/scripts");


# PROGRAMS
require "progs_lib.pl";
if (exists $ENV{"PROGS_DIR"}) {
    $prog_dir = $ENV{"PROGS_DIR"}."/";
} else {
    $prog_dir = "./";
}
#&read_progs(\%progs,"/home/phudelot/reduce_STRACK/progs.ini");
print "\nRead progs.ini \n\n";
&read_progs(\%progs, $prog_dir."progs.ini");

$dfits = $progs{"P_DFITS"};
$fitsort = $progs{"P_FITSORT"};
$awk = $progs{"P_GAWK"};
$stats = $progs{"P_STATS"};
$ic = $progs{"P_IC"};
$hough1_ref = $progs{"P_STRACK_HOUGH1"};
$hough1_ref_script = $progs{"S_STRACK_HOUGH"};
$hough_weight_ref = $progs{"P_STRACK_WEIGHT"};
$cphead = $progs{"P_CPHEAD"};
$getpix = $progs{"P_GETPIX"};

$weightdir = $ENV{"PIPESOFT"}."/stracktools/weight/";

#require "CFH12K_lib_port.pl";
require "instrument_lib.pl";

use Getopt::Long;

@ARGV_ref = @ARGV;

GetOptions("seg=s", \$seg,         
           "hweight", \$hweight,
           "hweight_only", \$hweight_only,
	   "hweight1=s", \$hweight1,
	   "hweight_fin=s", \$hweight_fin,
	   "out=s",\$out,
	   "hwmin=s",\$hwmin,
	   "hwmax=s",\$hwmax,
	   "hwcheck=s",\$hwcheck,
	   "hwtcheck=s",\$hwtcheck,
	   "hwscale=s",\$hwscale,
	   "hclip=s",\$hclip,
	   "bin=s",\$bin,
	   "refx=s",\$refx,
	   "refy=s",\$refy,
	   "check=s",\$check,
	   "noclean",\$noclean,
	   "instrum=s",\$instrum);

# Alternative ref px
$refx_alt = 1000;
$refy_alt = 0;

if (!defined $out) {
    $out = `echo $seg | sed -e 's/.fits//g'`; 
    chomp $out;
}  
if (!defined $check) {
    $check = 0.8;
}  
if (!defined $refx) {
    $refx = 0;
}
if (!defined $refy) {
    $refy = 0;
}
if (!defined $hweight1 && ($refx != 0 || $refy != 0)) {
    die "Need to supply hweight file for ref px other than 0,0 ... \n";
}

if ($refx == -1 && $refy == -1) {
    $refx = 0;
    $refy = 0;
    print "############################# \n";
    print " WARMING : ref point defaulted to (0,0) but track crosses the origin \n";
    print "############################# \n";
    undef $check;
}


# Tune the weight image
if (defined $hweight_fin && -e $hweight_fin) {
    print "\n\n Weight file $hweight_fin already present \n\n";
    $hweight1 = $hweight_fin;
} elsif (defined $hweight || defined $hweight1 || defined $hweight_only) {
    print "\n\n Use weight image ... \n\n";
    
    if (!defined $bin) {
	if (!defined $hweight1) {
	    $hweight1 = $weightdir."houghweight_".$instrum.".fits";
	    
	}
    } else {
	$binning = int($bin);
	$binning = "_b".$binning;
	if (!defined $hweight1) {
	    $hweight1 = $weightdir."houghweight_.".$instrum.$binning.".fits";
	}
    }
    
    if (defined $hweight_fin) {
	$hweight1 = $hweight_fin;
    }

    # If the weight file does not exist ---> create it
    print "Check for $hweight1 \n";
    if (!-e $hweight1 || defined $hwmax || defined $hwmin || defined $hwscale) {
	print "Create new weight image \n";

	#$weightfits = $weightdir."houghweight_".$instrum.".fits";
	$weightfits = $hweight1;
	$Nx_seg = `$dfits $seg | $fitsort -d NAXIS1 | $awk '{print \$2}'`+0.0;
	$Ny_seg = `$dfits $seg | $fitsort -d NAXIS2 | $awk '{print \$2}'`+0.0;
   
	print "hough_weight_ref $Nx_seg $Ny_seg $weightfits 0 0 \n";
	system("$hough_weight_ref $Nx_seg $Ny_seg $weightfits 0.0 0.0 \n");
    
    

    print "   weights : $hweight1 of instrument $instrum \n";
    

    # get min / max
    $line1 = `cat $hweight1 | $stats | grep min`;
    chomp $line1;
    $min1 = `echo "$line1" | $awk '{print \$3}' | sed -e 's/;//g'`+0.0;
    chomp $min1;
    $max1 = `echo "$line1" | $awk '{print \$6}'`+0.0;
    chomp $max1;
    
    #print "minmax $min1 $max1 \n";

    if (!defined $hwmax) {
	$hwmax1 = $max1;
    } else {
	$hwmax1 = $hwmax;
    }
    if (!defined $hwmin) {
	$hwmin1 = 300;
    } else {
	$hwmin1 = $hwmin;
    }
    if (!defined $hwscale) {
	$hwscale = 3;
    } 
    if (!defined $hwcheck) {
	$hwcheck = $hwmin;
    }
    if (!defined $hwtcheck) {
	$hwtcheck = 0.12;    # 7 deg
    }
    
    # scale it
    $new_weight1 = "weight1.fits.YY_".$$;
    $new2_weight1 = "weight1.fits.Y_rm ".$$;
    
    print "   Tune the weight file ... \n";
    print "ic '\%1 $hwscale /' $hweight1 > $new_weight1 \n";
    system("$ic '\%1 $hwscale /' $hweight1 > $new_weight1 \n");
    
    print "ic '\%1 $hwmin1 max  $hwmax1 min' $new_weight1 > $new2_weight1 \n";
    # system("$ic '\%1 $hwmin1 max $hwmax1 min' $new_weight1 > $new2_weight1 \n");
    system("$ic '\%1 $hwmin1 max $hwmax1 min' $new_weight1 > $hweight1 \n");
 
    system("rm -rf $new_weight1  \n ");
    
    #$hweight1 = $new2_weight1;
    
    }

    # if (defined $hweight_fin) {
    # 	system("cp $hweight1 $hweight_fin \n");
    # }
} else {
    print "How did you get there  ??? \n";
}

if (defined $hweight_only) {
    #print "test $new2_weight1 \n";
    if (defined $new2_weight1 && -e $new2_weight1) {
	system("rm -f $new2_weight1  \n ");
    }
    die;
}

# Build the raw hough transforms
$hough1 = $out."_h1_raw.fits";

print "\n\n#########################################################################################################\n";
print " HOUGH TRANSFORM : $hough1_ref $seg $hough1 0.5 $refx $refy \n";
print "#############################################################################################################\n";
system("$hough1_ref $seg $hough1 0.5 $refx $refy \n");

# clip and print result
if (defined $hclip) {
    print "\n\n Clip result \n";

    # clip it
    $hough1c = $out."_h1_c.fits";
    $hough1cc = $out."_h1_clip.fits";
    system("$ic '0 \%1 \%1 $hclip < ?' $hough1 > $hough1c \n");
    system("$ic '1 \%1 \%1 $hclip > ?' $hough1c > $hough1cc \n");
    system("rm -rf $hough1c \n");
}



# Use the weight images
if (defined $hweight) {
    
    print "Use weight image \n";

    # calculate Hough S/N
    $hough1_sn = $out."_hSN.fits";
    
    #$a = rfits $hough1;
    #$b = rfits $hweight1;
    #$c = $a / $b;
    #wfits $c,$hough1_sn; 
    
    #print "ic '\%1 \%2 /' $hough1 $hweight1 > $hough1_sn \n";
    system("$ic '\%1 \%2 /' $hough1 $hweight1 > $hough1_sn \n");
    #system("$ic '\%1 0 %2 0 != ?' $hough1_snn $hweight1 > $hough1_sn \n");
    
    # usual cphead
    #system("cphead $hough1 $hough1_sn REFX REFY \n");

    # my cphead
    system("echo REFX > listk.Z_".$$." \n");
    system("echo REFY >> listk.Z_".$$." \n");
    #print "$cphead $hough1 $hough1_sn \@listk.Z \n";
    system("$cphead $hough1 $hough1_sn \@listk.Z_".$$." \n");
    system("rm -f listk.Z_".$$." \n");
}

if (!defined $noclean) {
    @trash = ($new_weight1,$new2_weight1,$hough1);
    foreach (@trash) {
	if (defined $_ && -e $_) {
	    system("rm -f $_ \n");
	}
    }
    #system("rm -f $new_weight1 $new2_weight1 $hough1  \n");
} 


# Check if no track at the origin
if (defined $check) {

    print "\nCheck if track at the origin \n";

    $Nx_seg = `$dfits $seg | $fitsort -d NAXIS1 | $awk '{print \$2}'`+0.0;
    $Ny_seg = `$dfits $seg | $fitsort -d NAXIS2 | $awk '{print \$2}'`+0.0;
    
    # Possible alternative reference points
    my $Nxs = `$dfits $seg | $fitsort -d NAXIS1 | $awk '{print \$2}'`+0.0;
    my $Nys = `$dfits $seg | $fitsort -d NAXIS2 | $awk '{print \$2}'`+0.0;
    
    my @ref_alt = ();
    push(@ref_alt,[(0,0)]);
    push(@ref_alt,[(0,int(($Nys-1)/2.0))]);
    push(@ref_alt,[(int($Nxs/2),0)]);
    push(@ref_alt,[($Nxs-1,int(($Nys-1)/2.0))]);
    push(@ref_alt,[(0,$Nys-1)]);
    push(@ref_alt,[($Nxs-1,$Nys-1)]);
    push(@ref_alt,[(int($Nxs/2),$Nys-1)]);
    push(@ref_alt,[($Nxs,0)]);
    push(@ref_alt,[($Nxs,int(($Nys-1)/2.0))]);
    push(@ref_alt,[(int($Nxs/2.0),int($Nys/2.0))]);
    

    #$test = `$getpix_lim $hough1_sn $check 100000 1 10 1 $Ny \n`;
    $test = `$getpix -in $hough1_sn -vmin $check  -X 10  \n`;
    
    if (length($test) > 0) {
    	# Track at the origin ... Change the origin -> ($refx,$refy)
	print "     Change ORIGIN \n";
	
	# use next ref point
	my $ir = -1;
	my $iref = -2;
	foreach(@ref_alt) {
	    $ir++;
	    print "ref_alt $refx $refy  and $ir -> ".$ref_alt[$ir]->[0]." ".$ref_alt[$ir]->[1]."\n";
	    if ($ref_alt[$ir]->[0] == $refx && $ref_alt[$ir]->[1] == $refy) {
		if ($ir == $#ref_alt) {
		    $iref = -1;
		    last;
		} else {
		    $iref = $ir + 1;
		    last;
		}	       
	    }
	}
	if ($iref == -1) {
	    $refx_alt = -1;
	    $refy_alt = -1;
	} elsif ($iref == -2) { 
	    print "Big problem ... ref not found \n";
	    $refx_alt = 0;
	    $refy_alt = 0;
	} else {
	    $refx_alt = $ref_alt[$iref]->[0];
	    $refy_alt = $ref_alt[$iref]->[1];
	}	    
	
	my $nn = $#ref_alt;
	#print "test ref_old $refx $refy et $iref $refx_alt $refy_alt nref_alt $nn \n";
	#$test = <STDIN>;

	# Rebuid a weight file
	$weight_ref = $seg."_weight_ref.fits";
	#print "hough_weight_ref $Nx_seg $Ny_seg $weight_ref $refx_alt $refy_alt \n";
	system("$hough_weight_ref $Nx_seg $Ny_seg $weight_ref $refx_alt $refy_alt \n");
		
	$string = "perl $hough1_ref_script -hweight1 $weight_ref -refx $refx_alt -refy $refy_alt ";
	my $flag =0;
	foreach(@ARGV_ref) {
	    if ($flag == 1) {
		$flag = 0;
	    } elsif ($_ !~ /hweight1/ && $_ !~ /refy/ && $_ !~ /refx/) {
		$string .= $_." ";
	    } else {
		$flag = 1;
	    }
	}

	print $string."\n";
	system($string);

    }

}


