#!/usr/bin/env perl -w

############################################################################
# Prepare segmentation image for houg transform
# input : image   : image
#       : weight  : (def = root.weight.fits)
#       : seg     : segmentation image (def = root_SEG.fits)
#       : thresh  : detection threshold
#       : area    : area threshold
#       : instrum : instrument
#
# FILTERING PARAMETERS
#       : ell     : reject objects with ell < ellipticity
############################################################################

# HISTORY COMMENTS (TE):
#
# 19.12.2008:
# I cleaned up the code a bit
#
# 10.01.2013:
# I again started to clean up the code a bit

push(@INC, $ENV{'PERL_LIB'}) if (defined $ENV{'PERL_LIB'});
push(@INC, $ENV{"PIPESOFT"} . "/stracktools/scripts");

require "instrument_lib.pl";
require "progs_lib.pl";
use Getopt::Long;

print("Read progs.ini \n");

if (exists $ENV{"PROGS_DIR"}) {
  $prog_dir = $ENV{"PROGS_DIR"} . "/";
} else {
  $prog_dir = "./";
}

&read_progs(\%progs, $prog_dir . "progs.ini");

GetOptions(
  "image=s",   \$image,   "weight=s", \$weight, "seg=s", \$seg,
  "thresh=s",  \$thresh,  "area=s",   \$area,   "ell=s", \$ell,
  "instrum=s", \$instrum, "noclean",  \$noclean
);

die "Image missing \n" if (!defined $image);

$instrum = ""  if (!defined $instrum);
$thresh  = 1.1 if (!defined $thresh);
$ell     = 0.3 if (!defined $ell);
$area    = 5   if (!defined $area);

if (!defined $weight) {
  $weight = `echo $image | sed -e 's/.fits/.weight.fits/g'`;
  chomp $weight;
}

if (!defined $seg) {
  $seg = `echo $image | sed -e 's/.fits/_SEG.fits/g'`;
  chomp $seg;
}

@trash = ();

# PROGRAMS
$awk        = $progs{"P_GAWK"};
$sex        = $progs{"P_SEX"};
$p_filter   = $progs{"P_STRACK_FILTER2"};

# Run Sextractor
# create SExtractor configuration files on the fly:
&write_param($$);

$checkZ = "checkZ_" . $$ . ".fits";
$cat1   = "cat1.Z_" . $$;
push(@trash, $cat1);
push(@trash, $checkZ);
push(@trash, "default.sex_Z_" . $$);

print(
  "\n\n#############################################################################\n"
);
print("Get SEGMENTATION IMAGE \n");
print("$sex -c default.sex_Z_" 
    . $$
    . " $image -WEIGHT_IMAGE $weight  -CATALOG_NAME $cat1 -CHECKIMAGE_NAME $checkZ \n"
);
print(
  "#################################################################################\n"
);
system("$sex -c default.sex_Z_" 
    . $$
    . " $image -WEIGHT_IMAGE $weight  -CATALOG_NAME $cat1 -CHECKIMAGE_NAME $checkZ \n"
);

# filter the SEGMENTATION IMAGE
if (defined $ell) {
  print("\nFilter the segmentation image \n");

  open(IN, "default.param_Z_" . $$);

  # after the loop. $ip contains the column number of the
  # elliticity key in the SExtractor catalogue:
  $ip = 0;
  while (defined($oneline = <IN>)) {
    $ip++;
    if ($oneline =~ /ELLIPTICITY/) {
      last;
    }
  }
  close IN;

  $rem = "rem.Z_" . $$;
  push(@trash, $rem);
  system("$awk '\$1 != \"#\" && \$"
      . "$ip < $ell {print \$1,0,\$2,\$4,\$3,\$5}' $cat1 > $rem");
  $filter = "filterZ_" . $$ . ".fits";
  system("cp $checkZ $filter \n");
  system("$p_filter -in $filter -cat $rem -range \n");
} else {
  $filter = $checkZ;
}

system("mv $filter $seg \n");

# clean up temporary files:
if (!defined $noclean) {
  push(@trash, "weightZ.fits");
  push(@trash, $checkZ);

  foreach (@trash) {
    if (-e $_) {
      system("rm -f $_ \n");
    }
  }
}

#######################################################################
sub write_param {
  my ($pid) = @_;

  push(@trash, "default.param_Z_$pid");
  push(@trash, "default.nnw_Z_$pid");
  push(@trash, "default.conv_Z_$pid");

  open(OUT, ">default.param_Z_$pid");
  print OUT <<END;
NUMBER
XMIN_IMAGE
YMIN_IMAGE
XMAX_IMAGE
YMAX_IMAGE
ELLIPTICITY
END
  close OUT;

  open(OUT, ">default.conv_Z_$pid");
  print OUT <<END;
CONV NORM
# 3x3 ``all-ground'' convolution mask with FWHM = 2 pixels.
1 2 1
2 4 2
1 2 1
END
  close OUT;

  open(OUT, ">default.nnw_Z_$pid");
  print OUT <<END;
# Neural Network Weights for the SExtractor star/galaxy classifier (V1.3)
# inputs:       9 for profile parameters + 1 for seeing.
# outputs:      ``Stellarity index'' (0.0 to 1.0)
# Seeing FWHM range: from 0.025 to 5.5'' (images must have 1.5 < FWHM < 5 pixels)
# Optimized for Moffat profiles with 2<= beta <= 4.

 3 10 10  1

-1.56604e+00 -2.48265e+00 -1.44564e+00 -1.24675e+00 -9.44913e-01 -5.22453e-01  4.61342e-02  8.31957e-01  2.15505e+00  2.64769e-01
 3.03477e+00  2.69561e+00  3.16188e+00  3.34497e+00  3.51885e+00  3.65570e+00  3.74856e+00  3.84541e+00  4.22811e+00  3.27734e+00

-3.22480e-01 -2.12804e+00  6.50750e-01 -1.11242e+00 -1.40683e+00 -1.55944e+00 -1.84558e+00 -1.18946e-01  5.52395e-01 -4.36564e-01 -5.30052e+00
 4.62594e-01 -3.29127e+00  1.10950e+00 -6.01857e-01  1.29492e-01  1.42290e+00  2.90741e+00  2.44058e+00 -9.19118e-01  8.42851e-01 -4.69824e+00
-2.57424e+00  8.96469e-01  8.34775e-01  2.18845e+00  2.46526e+00  8.60878e-02 -6.88080e-01 -1.33623e-02  9.30403e-02  1.64942e+00 -1.01231e+00
 4.81041e+00  1.53747e+00 -1.12216e+00 -3.16008e+00 -1.67404e+00 -1.75767e+00 -1.29310e+00  5.59549e-01  8.08468e-01 -1.01592e-02 -7.54052e+00
 1.01933e+01 -2.09484e+01 -1.07426e+00  9.87912e-01  6.05210e-01 -6.04535e-02 -5.87826e-01 -7.94117e-01 -4.89190e-01 -8.12710e-02 -2.07067e+01
-5.31793e+00  7.94240e+00 -4.64165e+00 -4.37436e+00 -1.55417e+00  7.54368e-01  1.09608e+00  1.45967e+00  1.62946e+00 -1.01301e+00  1.13514e-01
 2.20336e-01  1.70056e+00 -5.20105e-01 -4.28330e-01  1.57258e-03 -3.36502e-01 -8.18568e-02 -7.16163e+00  8.23195e+00 -1.71561e-02 -1.13749e+01
 3.75075e+00  7.25399e+00 -1.75325e+00 -2.68814e+00 -3.71128e+00 -4.62933e+00 -2.13747e+00 -1.89186e-01  1.29122e+00 -7.49380e-01  6.71712e-01
-8.41923e-01  4.64997e+00  5.65808e-01 -3.08277e-01 -1.01687e+00  1.73127e-01 -8.92130e-01  1.89044e+00 -2.75543e-01 -7.72828e-01  5.36745e-01
-3.65598e+00  7.56997e+00 -3.76373e+00 -1.74542e+00 -1.37540e-01 -5.55400e-01 -1.59195e-01  1.27910e-01  1.91906e+00  1.42119e+00 -4.35502e+00

-1.70059e+00 -3.65695e+00  1.22367e+00 -5.74367e-01 -3.29571e+00  2.46316e+00  5.22353e+00  2.42038e+00  1.22919e+00 -9.22250e-01 -2.32028e+00

 0.00000e+00
 1.00000e+00
END
  close OUT;

  open(OUT, ">default.sex_Z_" . $pid);
  print OUT <<END;

# Default configuration file for SExtractor V1.2b14 - > 2.0
#
# (*) indicates parameters which can be omitted from this config file.

#-------------------------------- Catalog ------------------------------------

CATALOG_NAME    outfile        # name of the output catalog
CATALOG_TYPE    ASCII_HEAD      # "NONE","ASCII_HEAD","ASCII","FITS_1.0"
                                # or "FITS_LDAC"

PARAMETERS_NAME default.param_Z_$pid     # name of the file containing catalog contents

#------------------------------- Extraction ----------------------------------

DETECT_TYPE     CCD             # "CCD" or "PHOTO" (*)
FLAG_IMAGE      flag.fits       # filename for an input FLAG-image
DETECT_MINAREA  $area             # minimum number of pixels above threshold
DETECT_THRESH   $thresh          # <sigmas> or <threshold>,<ZP> in mag.arcsec-2
ANALYSIS_THRESH $thresh             # <sigmas> or <threshold>,<ZP> in mag.arcsec-2

FILTER          Y               # apply filter for detection ("Y" or "N")?
FILTER_NAME     default.conv_Z_$pid            # name of the file containing the filter

DEBLEND_NTHRESH 16              # Number of deblending sub-thresholds
DEBLEND_MINCONT 0.1             # Minimum contrast parameter for deblending

CLEAN           Y               # Clean spurious detections? (Y or N)?
CLEAN_PARAM     1.0             # Cleaning efficiency

MASK_TYPE       CORRECT         # type of detection MASKing: can be one of
                                # "NONE", "BLANK" or "CORRECT"

#------------------------------ Photometry -----------------------------------

PHOT_APERTURES  30               # MAG_APER aperture diameter(s) in pixels
PHOT_AUTOPARAMS 2.5, 3.5        # MAG_AUTO parameters: <Kron_fact>,<min_radius>

SATUR_LEVEL     50000          # level (in ADUs) at which arises saturation

MAG_ZEROPOINT   25.0            # magnitude zero-point
MAG_GAMMA       4.0             # gamma of emulsion (for photographic scans)
GAIN            7.0             # detector gain in e-/ADU.
PIXEL_SCALE     0.2               # size of pixel in arcsec (0=use FITS WCS info).

#------------------------- Star/Galaxy Separation ----------------------------

SEEING_FWHM     1.0           # stellar FWHM in arcsec
STARNNW_NAME    default.nnw_Z_$pid            # Neural-Network_Weight table filename

#------------------------------ Background -----------------------------------

BACK_SIZE       64              # Background mesh: <size> or <width>,<height>
BACK_FILTERSIZE 4               # Background filter: <size> or <width>,<height>

BACKPHOTO_TYPE  GLOBAL          # can be "GLOBAL" or "LOCAL" (*)
BACKPHOTO_THICK 24              # thickness of the background LOCAL annulus(*)

#------------------------------ Check Image ----------------------------------

CHECKIMAGE_TYPE SEGMENTATION    # can be one of "NONE", "BACKGROUND",
                                # "MINIBACKGROUND", "-BACKGROUND", "OBJECTS",
                                # "-OBJECTS", "SEGMENTATION", "APERTURES",
                                # or "FILTERED" (*)
CHECKIMAGE_NAME check.fits      # Filename for the check-image (*)

#--------------------- Memory (change with caution!) -------------------------

MEMORY_OBJSTACK 4000            # number of objects in stack
MEMORY_PIXSTACK 4000000          # number of pixels in stack
MEMORY_BUFSIZE  1024            # number of lines in buffer

#----------------------------- Miscellaneous ---------------------------------

VERBOSE_TYPE    NORMAL          # can be "QUIET", "NORMAL" or "FULL" (*)

#------------------------------- New Stuff -----------------------------------
WEIGHT_TYPE  MAP_WEIGHT
WEIGHT_IMAGE weight.fits
END
  close OUT;
}

