#!/usr/bin/perl 

# HISTORY COMMENTS:
#
# 10.01.2013:
# I started to clean up the code.
#
# 26.07.2014:
# function get_layout_pos:
# I updated the function to treat also the DECAM instrument.
# I had to 'formally' split DECAM chips in two chips in the
# vertical direction (very irregular layout of that instrument).

require "mathlib.pl";
require "progs_lib.pl";

push(@INC,$ENV{"PIPESOFT"}."/stracktools/scripts");

sub get_layout_pos {
    my ($chip,$layout) = @_;

    my @keys1 = keys(%{$layout});
    my @chiplist;

    foreach (@keys1) {
	my $k1 = $_;
	my @keys2 = keys(%{$layout->{$_}});
	foreach (@keys2) {
	    my $k2 = $_;
	    #print "k1k2 $k1 $k2 \n";
            $compchip = "$layout->{$k1}->{$k2}";
            $compchip =~ s/_.*//;
            $compchip_num = $compchip + 0;
	    if ($compchip_num == $chip) {
                push(@chiplist, ($k1, $k2)); 
	    } 
	}
    }

    if (0 + @chiplist > 0) {
        return @chiplist;
    } else {
        die "Problem ... no chip position for chip $chip \n";
    }
}

sub get_layout {
    my ($instrum,$layout) = @_;
	   
    my $confdir = $ENV{"PIPESOFT"}."/stracktools/conf/";
    my $file = $confdir."/".$instrum."_layout.ini";
    if (!-e $file) {
	die "Impossible to find $file \n";
    }

    my %data;
    read_catc($file,\%data);

    my $ii = -1;
    foreach(@{$data{"CHIP"}}) {
	$ii++;
	$xpos = $data{"XPOS"}->[$ii];
	$ypos = $data{"YPOS"}->[$ii];
	$layout->{$xpos}->{$ypos} = $_;
	#$test = $layout->{$xpos}->{$ypos};
	# print "test $test \n\n";
    }
    
    return 0;
}

sub get_code {
    my ($code) = @_;
    
    my %data = ();
    $data{"make_mask"} = "~/bin/make_mask";
    $data{"make_circle"} = "~/bin/make_circle";
    $data{"ww"} = "ww";
    
    if (!exists($data{$code})) {
    	return -1;	
    } else {
    	return $data{$code};	
    }
}

sub get_param {
    my ($instrument,$data) = @_;
    
    my $confdir = $ENV{"PIPESOFT"}."/stracktools/conf/";
    my $file = $confdir."instrum_conf.ini";
    my %dd;
    &read_catc($file,\%dd);
    
    my $index = -1;
    foreach(@{$dd{"INSTRUMENT"}}) {
        $index++;
        #print "instrum = $_ \n";
        if ($_ eq $instrument) {
    	last;
        }
    }
    
    if ($index == -1) {
        die "Instrument $instrument not found \n";
    }
    
    $data->{"NCHIPS"} = $dd{"NCHIPS"}->[$index];
    $data->{"MCHIPS"} = $dd{"MCHIPS"}->[$index];
    $data->{"REFX"} = $dd{"REFX"}->[$index];
    $data->{"REFY"} = $dd{"REFY"}->[$index];
    $data->{"PIXELSCALE"} = $dd{"PIXELSCALE"}->[$index];
    $data->{"XPIX_1"} = $dd{"XPIX_1"}->[$index];
    $data->{"XPIX_X"} = $dd{"XPIX_X"}->[$index];
    $data->{"YPIX_1"} = $dd{"YPIX_1"}->[$index];
    $data->{"YPIX_X"} = $dd{"YPIX_X"}->[$index];
}

sub get_period {
    my ($instrum,$id) = @_;

    #print "instrmid $instrum $id \n";

    my $confdir = $ENV{"PIPESOFT"}."/stracktools/conf/";
    my $file = $confdir."instrum_conf.ini";
    my %dd;
    &read_catc($file,\%dd);
    
    $ii = -1;
    foreach(@{$dd{"INSTRUMENT"}}) {
	$ii++;

	#print "instrum $_ \n";
	next if ($_ ne $instrum);

	# Get the GABODSID Range
	$range = $dd{"GID"}->[$ii];
	if ($range eq -1) {
	    return $dd{"PERIOD"}->[$ii];
	} else {
	    $range =~ /(.*)_(.*)/;
	    #print "range $1 $2 \n";
	    if ($id > $1 && $id < $2) {
		return $dd{"PERIOD"}->[$ii];
	    }
	}

    }
    die "BIG PROBLEM ... getting the period \n";

}

sub get_missing_chips {
    my ($instrument) = @_;

    my %dat = ();
    &get_param($instrument,\%dat);

    my @res = ();
    if ($dat{"MCHIPS"} eq "0") {
	return @res;
    } else {
	@res = split(/_/,$dat{"MCHIPS"});
    }
    return @res;
}


sub get_conf {
    my ($data) = @_;	
    $data->{"PIPESOFT"} = $ENV{"PIPESOFT"};
    $data->{"DATACONF"} = $data->{"PIPESOFT"}."/conf/reduction/"
}

1;


