#!/usr/bin/perl -w

# some functions to detect results from 'irregular' calculations:
sub isinf { $_[0]==9**9**9 || $_[0]==-9**9**9 }
sub isnan { ! defined( $_[0] <=> 9**9**9 ) }

# useful for detecting negative zero
sub signbit { substr( sprintf( '%g', $_[0] ), 0, 1 ) eq '-' }

# functions to give the minimum and maximum of two numbers:
sub max ($$) { $_[$_[0] < $_[1]] }
sub min ($$) { $_[$_[0] > $_[1]] }

###############################################################
# Is it in the list ?
###############################################################
sub is_in_num {
    my($list,$val) = @_;
    
    my $test = 0;
    foreach (@$list) {
        if ($_ == $val) {
            $test = 1;
            last;
        }
    }

    return $test;
}


###############################################################
# Grouping algorithm
# Simple algorithms ... all memebers of listxy will be grouped
###############################################################
sub group_simple {
    my($listxy,$tol,$groups) = @_;
    
    $tol2 = $tol * $tol;

    @$groups = ();
    $icurr = -1;
    my $ngroups = 0;
    foreach (@$listxy) {
	$icurr++;
	my $poscurr = $_;
	my $x = $_->[0];
	my $y = $_->[0];

	# test the groups
	my $ig = -1;
	my $isin = 0;
	foreach (@$groups) {
	    $ig++;
	
	    my $dist2 = gdist($poscurr,$groups,$ig);
	    #print "dist = $dist2 \n";
	    if ($dist2 <= $tol2) {             
		$isin = 1;
		# new element
		#print "New element \n";
		push(@{$groups->[$ig]->{'L'}},$icurr);
		
		# new center
		&new_center_groups($groups,$listxy,$ig);	    
	    }
	}
	# New group
	if ($isin == 0) {
	    $groups->[$ngroups]->{'C'}=[@$poscurr];
	    push(@{$groups->[$ngroups]->{'L'}},$icurr);
	    $ngroups++;
	}
    }
}

#############################################################################
# Grouping algorithm
# Members of gindexes will be groups / listxy contains all positions
# As output -> values of gindexes are listed in the groups_indexes structure
#############################################################################
sub group_simple_indexes {
    my($gindexes,$listxy,$tol,$groups_indexes) = @_;
    
    $tol2 = $tol * $tol;
    my @test = @$gindexes;
    
    if (defined $debug) {
	foreach (@$listxy) {
	    my @test = @$_;
	    print "listxy : @test \n";
	}
	my @test = @$gindexes;
	print "gindexxes = @test \n";
	print "tol = $tol \n";
    }

    @$groups_indexes = ();
    my $ngroups = 0;
    foreach (@$gindexes) {
	$curr = $_;
	my $poscurr = $listxy->[$curr];
	my $x = $listxy->[$curr]->[0];
	my $y = $listxy->[$curr]->[1];

	# test the groups
	my $ig = -1;
	my $isin = 0;
	foreach (@$groups_indexes) {
	    $ig++;
	
	    my $dist2 = gdist($poscurr,$groups_indexes,$ig);
	    if ($dist2 <= $tol2) {             
		$isin = 1;
		# new element
		#print "New element \n";
		push(@{$groups_indexes->[$ig]->{'L'}},$curr);
		
		# new center
		&new_center_groups($groups_indexes,$listxy,$ig);
		last;
	    }
	}
	# New group
	if ($isin == 0) {
	    $groups_indexes->[$ngroups]->{'C'}=[@$poscurr];
	    push(@{$groups_indexes->[$ngroups]->{'L'}},$curr);
	    $ngroups++;
	}
    }
}




#############################################################################
# Grouping algorithm
# Members of gindexes will be groups / listxy contains all positions
# As output -> values of gindexes are listed in the groups_indexes structure
# WARNING -> distance of a member to a group = distance to the clothest memeber
#         -> iterative
#############################################################################
sub group_indexes {
    my($gindexes,$listxy,$tol,$groups_indexes) = @_;
    
    my $tol2 = $tol * $tol;

    # First a simple group
    group_simple_indexes($gindexes,$listxy,$tol,$groups_indexes);

    #print "\n\n RECHECK ... \n";
    #foreach (@$groups_indexes) {
    #	print "New_group : "; 
    #	my @l = @{$_->{"L"}}; 
    #	print " @l \n";
    #}
    #print "\n\n";

    # Iterate to unify the groups
    my $ff = 1;
    while ($ff == 1) {
	# Number of groups
	my $ngrooups_ref = scalar(@$groups_indexes);
	#print "Number of groups = $ngrooups_ref \n";

	my @groups_list = ();
	# Loop on the groups
	my $ig = -1;
	foreach (@$groups_indexes) {
	    $ig++;
	    
	    # Loop on the elements
	    my $flag = 0;
	    foreach(@{$_->{"L"}}) {
		my $ind = $_;
		#print "Testing object $ind coord ".$listxy->[$ind]->[0]." ".$listxy->[$ind]->[1]."\n";

		# Check if present in the %groups_list
		my $tg = -1;
		foreach (@groups_list) {
		    $tg++;
		    
		    # Check if already present
		    if (is_in_num($_,$ind)) {
			#print " in the group $tg \n";
			$flag = 1;
			last;
		    }
		    
		    # Check if close to the groups
		    my @eq = ($listxy->[$ind]->[0],$listxy->[$ind]->[1]);
		    my $dist2 = gdist2_list(\@eq,\@groups_list,$tg,$listxy);
		    if ($dist2 <= $tol2) {
			#print " close to the group $tg \n";
			$flag = 1;
			last;
		    }
		
		    if ($flag == 1) {
			last;
		    }
		}

		# ALREADY EXISTING GROUP
		if ($flag == 1) {
		    #print "Already existing group \n";
		    foreach(@{$groups_indexes->[$ig]->{"L"}}) {
			$ind = $_;
			if (!is_in_num($groups_list[$tg],$ind)) {
			    push(@{$groups_list[$tg]},$_);
			}
		    }
		    last;
		}
	    }

	    # New group
	    if ($flag == 0) {
		#print "New group \n";
		$n = scalar(@groups_list);
		$groups_list[$n] = [@{$groups_indexes->[$ig]->{"L"}}];
	    }
	}

	# rebuild the groups
	@$groups_indexes = ();
	$ig = -1;
	foreach (@groups_list) {
	    $ig++;
	    $groups_indexes->[$ig]->{"L"} = [@$_];
	    &new_center_groups($groups_indexes,$listxy,$ig);
	}

	
	# Any change in the last iteration
	my $ngroups = scalar(@$groups_indexes);
	if ($ngroups == $ngrooups_ref) {
	    $ff = 0;
	}
    }

    # Check the result
    if (defined $debug) {
	print "\n\n RECHECK ... \n";
	foreach (@groups_list) {
	    print "New_group : "; 
	    foreach (@$_) {
		print " $_ ";
	    }
	    print "\n";
	}
    }

    # Any multiobject group ?
    foreach (@$groups_indexes) {
	if (scalar(@{$_->{"L"}}) > 1) {
	    return 1;
	}
    }

    return 0;

}

#############################################################################
# Grouping algorithm
# Members of gindexes will be groups / listxy contains all positions
# As output -> values of gindexes are listed in the groups_indexes structure
# WARNING -> distance of a member to a group = distance of the clothest px to the clothest member
#         -> iterative
#
# global variables : sat_chip, Nx,Ny,period,drho,dtheta
#############################################################################
sub group_indexes_px {
    my($gindexes,$listxy,$listim,$lnumber,$tol,$groups_indexes) = @_;
    
    my $tol2 = $tol * $tol;

    # First a group
    group_indexes($gindexes,$listxy,$tol,$groups_indexes);

    if (defined $debug) {
	print "\n\n RECHECK ... \n";
	foreach (@$groups_indexes) {
	    print "New_group simple group : "; 
	    my @l = @{$_->{"L"}}; 
	    print " @l \n";
	}
	print "\n\n";
    }

    # DO THE GROUP_PX on single object groups
    my $ig = -1;
    foreach (@$groups_indexes) {
	$ig++;

	my $n = scalar(@{$_->{"L"}});
	next if (scalar(@{$_->{"L"}}) > 1);
	$ind = $_->{"L"}->[0];

	next if ($sat_area[$ind] > 300);

	$im = $listim->[$ind];
	@eq = $listxy->[$ind]; 
	$num = $lnumber->[$ind];

	# get the pixels
	#print "$getpix -in $im -v $num \n";
	my @list = `$getpix -in $im -v $num  \n`;
	my @data=();
	foreach(@list) {
	    my $line = $_;
	    if($line =~ /(\d+),(\d+).*=\s(\S+)/) {
		
		my $eq1 = ($1-1) * $drho;
		my $eq2 = ($2-1-$Ny/2+0.5)*$dtheta;
		
		push(@data,[($eq1,$eq2)]);
	    }
	}
	
	# Loop on the px
	$ref = $sat_ref[$ind];
	my $chip = $sat_chip[$ind];
	my %limits_chip = get_chip_limits_instrum_ind0($period,$chip,$instrum);
	my %limits_mos = get_chippix_mosaic_instrum($period,$chip,$instrum);
	my $flag = 0;
	my $igt = -1;
	foreach (@data) {
	    # trannsform in mos eq
	    my @tt = @$_;
	    #print "test eq @tt \n";
	    
	    # convert 
	    my @eq = ();
	    &trans_line(\%limits_chip,\%limits_mos,\@tt,\@eq,$ref);
	    my @px_eq = ($eq[0]/$drho,$eq[1]/$dtheta);

	    # Loop on the groups
	    $igt = -1;
	    foreach(@$groups_indexes) {
		$igt++;
		next if ($ig == $igt);

		my $dist2 = gdist2(\@px_eq,$groups_indexes,$igt,$listxy);
		if ($dist2 <= $tol2) {
		    $flag = 1;
		    last;
		}
		
	    }
	    if ($flag == 1) {last;}
	}

	if ($flag == 1) {
	    # Add element to the group
	    push(@{$groups_indexes->[$igt]->{"L"}},$ind);
	    new_center_groups($groups_indexes,$listxy,$igt);
	
	    # Remove old group
	    splice(@$groups_indexes,$ig,1);
	}
    }

    if (defined $debug) {
	print "\n\n RECHECK ... \n";
	foreach (@$groups_indexes) {
	    print "New_group simple group : "; 
	    my @l = @{$_->{"L"}}; 
	    print " @l \n";
	}
	print "\n\n";
    }

}

#############################################################################
# Grouping algorithm
# Members of gindexes will be groups / listxy contains all positions
# As output -> values of gindexes are listed in the groups_indexes structure
# WARNING -> distance of a member to a group = distance of the clothest px to the clothest member center 
#         -> iterative
#############################################################################
sub group_indexes_px_complete {
    my($gindexes,$listxy,$listim,$lnumber,$tol,$groups_indexes) = @_;
    
    my $tol2 = $tol * $tol;

    # First a simple group
    group_simple_indexes($gindexes,$listxy,$tol,$groups_indexes);

    if (defined $debug) {
	print "\n\n RECHECK ... \n";
	foreach (@$groups_indexes) {
	    print "New_group : "; 
	    my @l = @{$_->{"L"}}; 
	    print " @l \n";
	}
	print "\n\n";
    }

    # Iterate to unify the groups
    my $ff = 1;
    while ($ff == 1) {
	
	# Number of groups
	my $ngrooups_ref = scalar(@$groups_indexes);
	#print "Number of groups = $ngrooups_ref \n";

	my @groups_list = ();
	# Loop on the groups
	my $ig = -1;
	foreach (@$groups_indexes) {
	    $ig++;
	    
	    # Loop on the elements
	    my $flag = 0;
	    foreach(@{$_->{"L"}}) {
		my $ind = $_;
		my $im = $listim->[$ind]; 
		my $num = $lnumber->[$ind];
		#print "Testing object $ind coord ".$listxy->[$ind]->[0]." ".$listxy->[$ind]->[1]."\n";

		# Check if present in the %groups_list
		my $tg = -1;
		foreach (@groups_list) {
		    $tg++;
		    
		    # Check if already present
		    if (is_in_num($_,$ind)) {
			#print " in the group $tg \n";
			$flag = 1;
			last;
		    }
		    
		    # Check if close to the groups
		    my @eq = ($listxy->[$ind]->[0],$listxy->[$ind]->[1]);
		    my $dist2 = gdist2_list(\@eq,\@groups_list,$tg,$listxy);
		    if ($dist2 <= $tol2) {
			#print " close to the group $tg \n";
			$flag = 1;
			last;
		    } elsif ($dist2 <= 3.0*$tol2) {
			# check with each px of the track
			
			# get the pixels
			#print "$getpix -in $im -v $num \n";
			my @list = `$getpix -in $im -v $num  \n`;
			my @data=();
			foreach(@list) {
			    my $line = $_;
			    if($line =~ /(\d+),(\d+).*=\s(\S+)/) {
				
				my $eq1 = ($1-1) * $drho;
				my $eq2 = ($2-1-$Ny/2+0.5)*$dtheta;
				
				push(@data,[($eq1,$eq2)]);
			    }
			}

			# Loop on the px
			$ref = $sat_ref[$ind];
			my $chip = $sat_chip[$ind];
			my %limits_chip = get_chip_limits_instrum_ind0($period,$chip,$instrum);
			my %limits_mos = get_chippix_mosaic_instrum($period,$chip,$instrum);
			foreach (@data) {
			    # trannsform in mos eq
			    my @tt = @$_;
			    
			    # convert 
			    my @eq = ();
			    &trans_line(\%limits_chip,\%limits_mos,\@tt,\@eq,$ref);
			    my @px_eq = ($eq[0]/$drho,$eq[1]/$dtheta);

			    my $dist2 = gdist2_list(\@px_eq,\@groups_list,$tg,$listxy);
			    
			    if ($dist2 <= $tol2) {
				$flag = 1;
				last;
			    }
			}

		    }
		
		    if ($flag == 1) {
			last;
		    }
		}

		# ALREADY EXISTING GROUP
		if ($flag == 1) {
		    #print "Already existing group \n";
		    foreach(@{$groups_indexes->[$ig]->{"L"}}) {
			$ind = $_;
			if (!is_in_num($groups_list[$tg],$ind)) {
			    push(@{$groups_list[$tg]},$_);
			}
		    }
		    last;
		}
	    }

	    # New group
	    if ($flag == 0) {
		#print "New group \n";
		$n = scalar(@groups_list);
		$groups_list[$n] = [@{$groups_indexes->[$ig]->{"L"}}];
	    }
	}

	# rebuild the groups
	@$groups_indexes = ();
	$ig = -1;
	foreach (@groups_list) {
	    $ig++;
	    $groups_indexes->[$ig]->{"L"} = [@$_];
	    &new_center_groups($groups_indexes,$listxy,$ig);
	}

	# Any change in the last iteration
	my $ngroups = scalar(@$groups_indexes);
	if ($ngroups == $ngrooups_ref) {
	    $ff = 0;
	}
    }

    # Check the result
    if (defined $debug) {
	print "\n\n RECHECK ... \n";
	foreach (@groups_list) {
	    print "New_group : "; 
	    foreach (@$_) {
		print " $_ ";
	    }
	    print "\n";
	}
    }

    # Any multiobject group ?
    foreach (@$groups_indexes) {
	if (scalar(@{$_->{"L"}}) > 1) {
	    return 1;
	}
    }

    return 0;

}


#####################################################################
# Distance of a member to a group center
#####################################################################
sub gdist {
    my ($mem,$groups,$ig) = @_;
    
    my @geq = @{$groups->[$ig]->{'C'}}; 

    $dist = ($mem->[0] - $geq[0])*($mem->[0] - $geq[0]) + ($mem->[1] - $geq[1])*($mem->[1] - $geq[1]); 

    return $dist;
}

#####################################################################
# Distance of a member to a group = distance to the clothest member
#####################################################################
sub gdist2 {
    my ($mem,$groups,$ig,$listxy) = @_;
    $dist  = 1e60;

    foreach (@{$groups->[$ig]->{'L'}}) {
	my @geq = @{$listxy->[$_]};
	my $dist2 = ($mem->[0] - $geq[0])*($mem->[0] - $geq[0]) + ($mem->[1] - $geq[1])*($mem->[1] - $geq[1]); 
	if ($dist2 < $dist) {
	    $dist = $dist2;
	}
    }
    return $dist;
}

#####################################################################
# Distance of a member to a group = distance to the clothest member (groups_list)
#####################################################################
sub gdist2_list {
    my ($mem,$groups,$ig,$listxy) = @_;
    $dist  = 1e60;

    foreach (@{$groups->[$ig]}) {
	my @geq = @{$listxy->[$_]};
	my $dist2 = ($mem->[0] - $geq[0])*($mem->[0] - $geq[0]) + ($mem->[1] - $geq[1])*($mem->[1] - $geq[1]); 
	if ($dist2 < $dist) {
	    $dist = $dist2;
	}
    }
    return $dist;
}

#####################################################################
# Get the new center of a group
#####################################################################
sub new_center_groups {
    my ($groups,$listxy,$ig) = @_;

    my @res = ();
    my @res2 = ();
    
    my $n = 0;
    my $gr = $groups->[$ig];
    foreach (@{$gr->{'L'}}) {
	$n++;
	my @eq = @{$listxy->[$_]};
	$res[0] += $eq[0];
	$res[1] += $eq[1];
	
	$res2[0] += $eq[0]*$eq[0];
	$res2[1] += $eq[1]*$eq[1];
	#print "test @eq \n";
    }

    if ($n == 0) {
	die "Big error : empty group \n";
    }

    $res[0] /= $n;
    $res[1] /= $n;

    $res2[0] /= $n;
    $res2[1] /= $n;
    $res2[0] = sqrt($res2[0] - $res[0]*$res[0]);
    $res2[1] = sqrt($res2[1] - $res[1]*$res[1]);
 
    $gr->{'C'} = [@res];
    $gr->{'D'} = [@res2];
}

############################################
# Small mathematical library
############################################


#############################################
# Read col of a file
#############################################
sub read_col { 
	($file,$col,$res) = @_;	
	$min = 1e32;
	$max = -1e32;
	
	open(IN,$file) || die "Impossible to open $file \n";
	while(defined($oneline = <IN>)) {
		$oneline =~ /\s*(\S.*)/;
		my @list = split(/\s+/,$1);
		#print "Newline $oneline \n";
		$val = $list[$col-1];
		push(@$res,$val);
		if ($val < $min) {
			$min = $val;	
		} elsif ($val > $max) {
			$max = $val;	
		}
	}
	close IN;
	
	return ($min,$max);
}
#############################################
# Read col of a file + sort
#############################################
sub read_col_sort {
	($file,$col,$res) = @_;	
	$min = 1e32;
	$max = -1e32;
	
	
	my @xx = ();
	open(IN,$file) || die "Impossible to open $file \n";
	while(defined($oneline = <IN>)) {
		$oneline =~ /\s*(\S.*)/;
		my @list = split(/\s+/,$1);
		push(@xx,$list[$col-1]);
	}
	close IN;
	
	@$res = sort by_number @xx;
	
	return ($min,$max);
}
#############################################
# Get the mean
#############################################
sub mean {
	my $tmp = 0;
	my @list = @_;	
	foreach (@list) {
		$tmp += $_;	
	}
	$tmp /= scalar(@list);
	return $tmp;
}
#############################################
# Get the stddev
#############################################
sub stddev {
	my $sum = 0;
	my $ssum = 0;
	my @list = @_;	
	foreach (@list) {
		$sum += $_;
		$ssum += $_ * $_;
	}
	my $n = scalar(@list);
	$sum /= $n;
	my $tmp = sqrt(($ssum - $n * $sum * $sum) / ($n - 1.0));
	
	return $tmp;
}
#############################################
# Get the percentiles
#############################################
sub percent {
	my ($list,$min,$max) = @_;	
	
	if ($min > 1 || $min < 0 || $max > 1 || $max < 0) {
		die "Percentile limits incorrect $min $max \n";	
	}
	
	# Sort it
	my @sorted = sort by_number @$list;
	
	my $imin = int($min * scalar(@$list)+0.5);
	my $imax = int($max * scalar(@$list)+0.5);
	
	$pmin = $sorted[int($min * scalar(@$list)+0.5)];
	$pmax = $sorted[int($max * scalar(@$list)+0.5)];
	
	return (($pmin,$pmax));
}
#===============================================================================
sub by_number2 {         # See "sub median"...

    if ($b < $a) {
       return -1;
    } elsif ($a == $b) {
       return 0;
    } elsif ($b > $a) {
       return 1;
    }

}
#===============================================================================
sub by_number {         # See "sub median"...

    if ($a < $b) {
       return -1;
    } elsif ($a == $b) {
       return 0;
    } elsif ($a > $b) {
       return 1;
    }

}
#===============================================================================

sub mean_file {
	my ($file,$col) = @_;
	my @list = ();
	&read_col($file,$col,\@list);	
	
	return mean(@list);
}
#===============================================================================
sub stddev_file {
	my ($file,$col) = @_;
	my @list = ();
	&read_col($file,$col,\@list);	
	
	return stddev(@list);
}
#===============================================================================
sub percent_file {
	my ($file,$col,$min,$max) = @_;
	my @list = ();
	&read_col($file,$col,\@list);	
	
	return percent(\@list,$min,$max);
}
#===============================================================================

#####################
# Histogram routines
#####################
sub get_hist {
	
	my ($nbin,$x,$min,$max) = @_;	
	
	# Cell size
	#$cell = ($max - $min) / ($nbin - 1);
	$cell = ($max - $min) / $nbin;
	my @tr = ($min - $cell,$cell);
	
	@hist = ();
	&bin($nbin,$x,\@hist,\@tr);
	
	return @hist;
}
############################
sub bin {
	my 	($nbin,$x,$hist,$tr) = @_;
	$n = scalar(@$x);
	
	# Loop on the bins
	my $i = 0;
	foreach ($i = 0;$i < $nbin; $i++) {
		my $min = $tr->[0] + ($i+1) * $tr->[1];
		my $max = $min + $tr->[1];	
	
		#print "min=$min max=$max \n";
	
		push(@$hist, 0);
		$ii = 0;
		foreach(@$x) {
			$ii++;
			if ($i == 0 && $_ >= $min && $_ <= $max) {
				$hist->[$i] += 1;	
			} elsif ($_ > $min && $_ <= $max) {
				$hist->[$i] += 1;	
			}
		}
		$hist->[$i] /= $n * $tr->[1];	
	}
}
#############################


###########################
# Regridding //// CHECK INDEXES (FORTRAN TAB BEGINS WITH INDEX 1)
###########################
sub interp1D {
	my ($nx,$nxin,$image,$xp,$xnew,$iw) = @_;


	my $flag = 0;
	my $xoldstep = $xp->[1] - $xp->[0];
	my $i = int(($xnew-0.5)*$nxin/$nx +0.5) - 1;
	my $t = ($xnew - $xp->[$i]) / $xoldstep;

	$tmp2 = $xp->[$i];
	#print "xp = @xp \n";
	#my @tmp = @$image;
	#print "image = @tmp \n";
	#die;
	
	#print "test = $xnew $i $t \n"; 
	
	if ($i < 0) {
		$flag = 1;
		$i = 0;	
	} elsif ($i >= $nxin-1) {
		$flag = 1;
		$i = $nxin - 2;	
	}

	#print "test = $xnew $i $t \n"; 

	########## Linear interp #########
	if ($iw == 0) {
		if ($flag == 1) {
			return $image->[$i];
		} else {
			my $bl = $image->[$i];
			my $br = $image->[$i+1];
			$res = (1-$t) * $bl + $t * $br;
			#print "bl/br $bl and $br and $res \n";
			
			return (1-$t) * $bl + $t * $br;
		}
	###### nearest neighbour #########
	} else {
		if ($flag == 1) {
			return $image->[$i];
		} else {
			my $bl = $image->[$i];
			my $br = $image->[$i+1];
			if ($t < 0.5) {
				return $bl;	
			} else {
				return $br;	
			}
		}	
	}
	die "How did you get there ??\n";
}
###########################
sub regrid1D {
	my($nx,$nxin,$nxout,$arrin,$arrout,$xp,$iw) = @_;	
	my $i;
	
	# If same size
	if ($nxin == $nxout) {
		foreach(@$arrin) {
			push(@$arrout,$_);	
		}	
		return;
	}
	
	@xp = ();
	for($i=0;$i<$nxin;$i++) {
		$val = 0.5 + (2*$i +1)/2 * $nx / $nxin;
		push(@xp,$val);	
	}
	
	# interpolate the array
	for($i=0;$i<$nx;$i++) {
		if ($i <= $nxout) {
			$xnew = 0.5 + (2*$i +1)/2 * $nx / $nxout;
			my $value = interp1D($nx,$nxin,$arrin,\@xp,$xnew,0); 
			push(@$arrout,$value);
		} else {
			push(@$arrout,0);
		}
		#print "test $i $xnew  ".$arrout->[$i]."\n";
	}
}

# ####################################
# # Smoothing routines
# ####################################
# sub smoothhist {
# 	my ($nx,$array,$cell,$fwhm,$smoothcrop) = @_;
# 	
# 	my $nx2 = 256; my $nx3 = 2*$nx2;
# 	
# 	# Size of kernel image + new cell size
# 	my $npix = int(($fwhm*$nx2)/($cell*$nx));
# 	my $sell = $cell * $nx / $nx2;
# 		
# 	
# 	# Build working array (twice the size + padding with 0)
# 	my @array2 = (); 
# 	#my @smootharray = ();
# 	for($i = 0;$i<$nx/2;$i++) {
# 		push(@array2,0);	
# 	} 
# 	for($i = $nx/2;$i<3*$nx/2;$i++) {
# 		push(@array2,$array->[$i-$nx/2]);	
# 	} 
# 	for($i = 3*$nx/2;$i<$nx*2;$i++) {
# 		push(@array2,0);	
# 	} 
# 	
# 	# Smoothing method + go
# 	$iw = 0;
# 	$nnx = 2 * $nx;
# 	&smooth1D(\@array2,$nnx,\@smootharray,$nx3,$npix,$iw);
# 	
# 	
# 	# normalize it
# 	my $sum = 0;
# 	for ($i = 0;$i < $nx3; $i++) {
# 		$sum += $smootharray[$i] * $sell;
# 		#print "sum = $sum \n";	
# 	}
# 	for ($i = 0;$i < $nx3; $i++) {
# 		$smootharray[$i] /= $sum;	
# 	}
# 	
# 	
# 	# crop it
# 	$x1 = $nx2/2; $x2= 3 * $nx2 / 2;
# 	for ($i = $x1;$i< $x2;$i++) {
# 		push(@$smoothcrop,$smootharray[$i]);	
# 		#print TEST "$i ".$smootharray[$i]."\n";
# 	}
# 	#close TEST;
# }

#############################
# Make the gaussian
############################# 
sub makebeam {
	my ($beam,$nx,$nxout,$barea,$bfwhm) = @_;
	
	for ($i=0;$i<$nx;$i++) {
		$beam->[$i]=0;	
	}
	
	my $centrex = $nxout / 2;
	my $sigma = $bfwhm / (2*sqrt(2*log(2)));
	
	$barea = 0;
	my $sig = 2 * $sigma * $sigma;
	for ($i = 0; $i < $nxout; $i++) {
		$dist = abs($i-$centrex);
		$beam->[$i] = exp(-$dist*$dist/$sig) ;
		$barea += $beam->[$i];	
		#print "fwhm $i ".$beam->[$i]."\n";
	}
	
}
sub makebeam_wrap {
	my ($beam,$nx,$nxout,$barea,$bfwhm) = @_;
	
	#my $centrex = $nxout / 2;
	my $sigma = $bfwhm / (2*sqrt(2*log(2)));
	
	$barea = 1;
	my $sig = 2 * $sigma * $sigma;
	
	$beam->[0] = 1;
	for ($i = 1; $i < $nxout/2; $i++) {
		$beam->[$i] = exp(-$i*$i/$sig);
		$beam->[$nxout-1-$i] = exp(-$i*$i/$sig);
		$barea += 2 * $beam->[$i];	
		#print "fwhm $i ".$beam->[$i]."\n";
	}
	
	# normalize
	for ($i=0;$i<$nxout-1;$i++) {
		$beam->[$i] /= $barea;	
	}
	
	@test = @$beam;
	#print "beam2 = @test \n";
	
}

###############################
# Smoothing in 1D
###############################
# sub smooth1D {
# 	my ($small,$nxin,$arrayb,$nx,$npix,$iw) = @_;
# 	
# 	# Initialize
# 	my $nxout = $nx;
# 	my $bfwhm = $npix;
# 	my $ndim = 1;
# 	my @nn = ($nx);
# 	
# 	# Regrid the array
# 	@large = ();
# 	for ($i = 0;$i < $nx; $i++) {
# 	#	push(@large,0);
# 	}
# 	for ($i = 0;$i < $nxin; $i++) {
# 		$large[$i] = $small->[$i];
# 	}
# 	
# 	my @xp;
# 	#print "$nx,$nxin,$nxout,$iw \n\n";
# 	#print "large = @large \n";
# 	my @arrayc = ();
# 	&regrid1D($nx,$nxin,$nxout,\@large,\@arrayc,\@xp,$iw);
# 	
# 	# Build the smoothing kernel
# 	my @beam; my $barea;
# 	&makebeam_wrap(\@beam,$nx,$nxout,$barea,$bfwhm);
# 	
# 	
# 	# Create the FFT object
# 	$fft = new Math::FFT(\@arrayc);
# 	
# 	# convolve with teha gaussian kernel
# 	$test = $fft->convlv(\@beam);
# 	
# 	@test = @$test;
# 	#print_list2(@test);
# 	#print "test = @test \n";
# 
# 	foreach(@test) {
# 		push(@$arrayb,$_);	
# 	}
# }

####################################################
# Find peak of an histogram
# tr[0] = min-cell tr[1] = cell
####################################################
sub findpeak {
	my ($hist,$tr,$imax,$vmax) = 	@_;
	
	my $max = -1e32;
	my $ii = -1;
	$imax = 0; 
	
	foreach(@$hist) {
		$ii++;
		if ($_ > $max) {
			$imax = $ii;
			$max = $_;	
		}
	}
	
	$vmax = $tr->[0] + ($imax+1.5) * $tr->[1];
	return $vmax;
}

####################################################
# Plot histogram
# tr[0] = min-cell tr[1] = cell
####################################################
sub plot_histo {
	my ($hist,$tr) = 	@_;
	
	my $min = $tr->[0];
	
	
	my @xx = (); 
	my @yy = ();
	
	$x = $min;
	foreach(@$hist) {
		$x += $tr->[1];
		push(@xx,$x);
		push(@yy,$_);
	}
	
	return (\@xx,\@yy);
	
}

###########################################
# Find the levels
# levels has to be ordered (increasing)
###########################################
sub get_levels {
	my ($cell,$inlevels,$outlevels,$z) = @_;
	
	# copy the table + normalize
	my @copy = ();
	my $sum = 0;
	foreach(@$z) {
		push(@copy,$_);
		$sum += $_;
	}
	$sum *= $cell;
	
	my $i = -1;
	foreach(@copy) {
		$i++;	
		$copy[$i] /= $sum;
	}
	
	
my @scopy = sort by_number2 @copy;
	
	
	# Look for the levels
	my $current = 1;
	my $sum2 = 0;
	$ii = -1;
	foreach(@scopy) {
		$ii++;
		$sum2 += $_*$cell;
				
		# print "$ii $_ $sum2 \n";
		
		
		my $il = 0;
		foreach (@$inlevels) {
			$il++;
			if ($sum2 > $_ && $current == $il) {
				$outlevels->[$il-1] = $scopy[$ii] * $sum;
				#print "found $il ".$scopy[$ii]."\n";
				$current++;	
			}
		}
		if ($current > scalar(@$inlevels)) {
			last;	
		}
	}

}
###########
# get plevels
# Z not sorted
# INPUT LEVELS (levels) have to be ordered !!!!!!!!!!!!!!!!!!
###########
sub get_plevels {
	my ($z,$tr,$levels,$p1,$p2) = @_;	
	
	my $sell = $tr->[1];
	my $tr0 = $tr->[0];
	my $nlevels = scalar(@$levels);
	
	my $current = 1;
	my $ii = -1;
	my @data = ();
	foreach(@$z) {
		$ii++;
		my $val = $_;
		
		my $il = -1;
		foreach (@$levels) {
			$il++;
			if ($val > $_) {
				push(@{$data[$il]},$ii);	
			}	
		}
	}
	
	my $il = -1;
	foreach (@$levels) {
		$il++;
		my $ival1 = $data[$il]->[0] + 1;
		my $ival2 = pop( @{$data[$il]} ) + 1;
		my $val1 = $tr0 + $ival1 * $sell;
		my $val2 = $tr0 + $ival2 * $sell;
		
		push(@$p1, $val1 );	
		push( @$p2 , $val2 );
	}
}
sub get_plevels_phil {
	my ($z,$tr,$levels,$p1,$p2) = @_;	
	
	my $sell = $tr->[1];
	my $tr0 = $tr->[0];
	my $nlevels = scalar(@$levels);
	
	
	my $current = 1;
	my $ii = -1;
	my @data = ();
	my @data_flag = ();
	
	foreach (@$levels) {
		push(@data_flag,"NaN");	
	}
	
	foreach(@$z) {
		$ii++;
		my $val = $_;
		
		my $il = -1;
		foreach (@$levels) {
			$il++;
			
			if ($data_flag[$il] == 1 && $val < $_) {
				$data_flag[$il] = -1;
			}
			
			if ($data_flag[$il] != -1 && $val > $_) {
				push(@{$data[$il]},$ii);
				$data_flag[$il] = 1;	
			}	
		}
	}
	
	my $il = -1;
	foreach (@$levels) {
		$il++;
		my $ival1 = $data[$il]->[0] + 1;
		my $ival2 = pop( @{$data[$il]} ) + 1;
		my $val1 = $tr0 + $ival1 * $sell;
		my $val2 = $tr0 + $ival2 * $sell;
		
		push(@$p1, $val1 );	
		push( @$p2 , $val2 );
	}
}







1;
