#!/usr/bin/perl -w


use PGPLOT;
use PDL;
use PDL::Graphics::PGPLOT::Window;
use PDL::Image2D;

############################################################ 
# Plot a reg file on a chip
# input : image : image name
#       : out  : otput ps file
#       : reg  : reg name 
############################################################

use Getopt::Long;
GetOptions("image=s", \$im,          
           "reg=s", \$reg,          
           "out=s", \$out,         
           "noreg", \$noreg,
	   "bin=s",\$bin,
	   "noprint",\$noprint,
	   "conv=s",\$conv);   


$homedir = $ENV{'HOME'};
#push(@INC,"$homedir/perl/VIRMOS/pipeline");
push(@INC,$ENV{"PIPESOFT"}."/stracktools/scripts");

require "progs_lib.pl";

# Read progs.ini file
if (exists $ENV{"PROGS_DIR"}) {
    $prog_dir = $ENV{"PROGS_DIR"}."/";
} else {
    $prog_dir = "./";
}
#&read_progs(\%progs,"/home/phudelot/reduce_STRACK/progs.ini");
&read_progs(\%progs,$prog_dir."progs.ini");

$dfits = $progs{"P_DFITS"};
$fitsort = $progs{"P_FITSORT"};
$awk = $progs{"P_GAWK"};


# General param
$percent = 2.5;
$marg = 30;

if (!defined $bin) {$bin = 1;}
if (!defined $conv) {$conv = 0;}

push(@files,$im);

# Get parameters from the files
@lines = `$dfits $im | $fitsort -d CRPIX1 CRPIX2 CRVAL1 CRVAL2 CD1_1 CD1_2 CD2_1 CD2_2`;
foreach (@lines) {
    @list = split(/\s/,$_);
    push(@CRPIX1,$list[1]);
    push(@CRPIX2,$list[2]);
    push(@CRVAL1,$list[3]);
    push(@CRVAL2,$list[4]);
    push(@CD1_1,$list[5]);
    push(@CD1_2,$list[6]);
    push(@CD2_1,$list[7]);
    push(@CD2_2,$list[8]);
}

# Get the layout
$layout[0]->[0]=0;
$layout[0]->[1]=0;

$nx = 1;
$ny = 1;

# prepare the environment
my @xrange = ();
my @yrange = ();

@{$env_range[0]->[0]} = (0,1,0,1);

@tmp = @{$env_range[1]->[1]};
print "@tmp \n";


# get the size of the total mosaic
$im0 = $files[0];
$line = `$dfits $im0 | $fitsort -d NAXIS1 NAXIS2`;
$line =~ /(\S*)\s+(\S*)\s+(\S*)/;
$naxis1 = $2;
$naxis2 = $3;
$size1 = $nx * $naxis1;
$size2 = $ny * $naxis2;


# Plot it
if ($size1/$size2 > 30/20) {
    $size = 280;
    @size = ($size,$size*$size2/$size1);
} else {
    $size = 200;
    @size = ($size*$size1/$size2,$size);
}
#$size = 4;
#@size = ($size,$size*$size2/$size1);
print "size12 $size1 $size2 @size \n";
if (defined $noprint) {
    $win = PDL::Graphics::PGPLOT::Window->new({Dev=>'/xserve',Size=>\@size,Unit=>2});
} else {
    $dev = "\"".$out."\"/PS";
    $win = PDL::Graphics::PGPLOT::Window->new({Dev=>$dev,Size=>\@size,Unit=>2});
}
#$win = PDL::Graphics::PGPLOT::Window->new({Dev=>'/xserve',Size=>\@size});

# Loop on the chips
$ii = -1;
foreach (@layout) {
    $ii++;

    $name = $files[$ii];
    $layx = $layout[$ii]->[0];
    $layy = $layout[$ii]->[1];
    
    @range = @{$env_range[$layx]->[$layy]};
    $name2 = $name;
    &plot_fits($win,$name2,\@range,$bin,$conv);


    # Read reg file if given
    if (defined $reg) {
	
	$name = $reg;

	# read it
	my (@lx,@ly);
	if (-e $name) {
	    &read_reg($name,\@lx,\@ly,$bin);
	}
    
	# plot it
	$ir = -1;
	foreach (@lx) {
	    $ir++;
	    my @x = @{$lx[$ir]};
	    my @y = @{$ly[$ir]};
	    push(@x,$x[0]);
	    push(@y,$y[0]);
	    print "plot reg @x @y \n";


	    #my $x = pdl @x; my $y = pdl @y;
	    $win->line(\@x,\@y,{PlotPosition => [$range->[0],$range->[1],$range->[2],$range->[3]]});
	}

    }
    
    
    
}


###############################################################
sub plot_fits {
    my ($win,$im,$range,$bin,$conv) = @_;

    my $array3 = rfits $im;

    if ($conv > 1) {
	$array = conv2d $array3, ones($conv,$conv), {Boundary => Reflect};	
    } else {
	$array = $array3;
    }

    # get the size
    my $sizex = $array3->hdr->{NAXIS1};
    my $sizey = $array3->hdr->{NAXIS2};
    
    # rebin it 
    if ($bin > 1) {
	$sizex /= $bin;
	$sizey /= $bin;
	$sizex = int($sizex);
	$sizey = int($sizey);
	$array2 = rebin($array,$sizex,$sizey);
    } else {
	$array2 = $array;
    }

    $f = $array2->flat;
    my $nelem = nelem($f);
   
    # get the scale from a slice on the data
    if ($nelem > 2000000) {
	$s = $f->mslice([$nelem/2-1e6,$nelem/2+1e6]);
	$qs = $s->qsort;
    } else{
	$qs = $f->qsort;
    }
    $nelem = nelem($qs);
    
    $max = $qs->index($percent/100*$nelem);
    $min = $qs->index((1-$percent/100)*$nelem);

    #print "min $min max $max size $sizex $sizey \n";

    print $range->[0]." ".$range->[1]." ".$range->[2]." ".$range->[3]."\n";
    $win->env(0, $sizex-1, 0, $sizey-1,1,-1,{PlotPosition => [$range->[0],$range->[1],$range->[2],$range->[3]]});
    $win->imag( $array2 ,$min,$max,{DrawWedge=>0,PlotPosition => [$range->[0],$range->[1],$range->[2],$range->[3]]});

}
###############################################################
sub read_reg {
    my ($name,$lx,$ly,$bin) = @_;

    $ir = -1;
    open(REG,$name) || die "Impossible to open $name \n";
    while(<REG>) {
	$line = $_;
	if ($line !~ /polygon/ && $line !~ /POLYGON/) {next;}
	if($line =~ /\((.*)\)/) {
	    $ir++;
	    @list = split(/\,/,$1);
	    
	    $flag = 0;
	    foreach(@list) {
		if ($flag == 0) {
		    push(@{$lx->[$ir]},$_/$bin);
		    $flag = 1;
		} else {
		    push(@{$ly->[$ir]},$_/$bin);
		    $flag = 0;
		}
	    }
	}
    }

}
###############################################################

