#!/usr/bin/perl -w


use PGPLOT;
use PDL;
use PDL::Graphics::PGPLOT::Window;
use PDL::ImageND;
use PDL::Image2D;

############################################################ 
# Plot a reg file on a mosaic
# input : root : root name of the mosaic
#       : out  : output ps file
#       : root_reg  : reg root name (def = root) 
#       : ext  : extentions (ex: OFCS.fits)
#       : ext_reg : extention for reg file (def = none)
#       : noreg : witout reg files
#       : bin   : binning of the fits files
############################################################

use Getopt::Long;
GetOptions("root=s", \$root,          
           "root_reg=s", \$root_reg,          
           "ext=s", \$ext,          
           "out=s", \$out,         
           "noreg", \$noreg,
	   "conv=s", \$conv,
	   "noprint", \$noprint,
	   "bin=s",\$bin);   

$homedir = $ENV{'HOME'};
#push(@INC,"$homedir/perl/VIRMOS/pipeline");
push(@INC,$ENV{"PIPESOFT"}."/stracktools/scripts");

# Read progs.ini file
require "progs_lib.pl";
if (exists $ENV{"PROGS_DIR"}) {
    $prog_dir = $ENV{"PROGS_DIR"}."/";
} else {
    $prog_dir = "./";
}
#&read_progs(\%progs,"/home/phudelot/reduce_STRACK/progs.ini");
print "\n Read progs.ini ...\n";
&read_progs(\%progs,$prog_dir."progs.ini");

$dfits = $progs{"P_DFITS"};
$fitsort = $progs{"P_FITSORT"};
$awk = $progs{"P_GAWK"};

# General param
$percent = 2.5;
$marg = 30;

if (!defined $bin) {$bin = 1;}
if (!defined $conv) {$conv = 0;}

# get the files
if (defined $ext) {
    @files = glob($root."*".$ext);
    $regname = $root."*".$ext;
} else {
    @files = glob($root."*");
    $regname = $root."*";
    $ext = "";
}

# get chip numbers
@chips = ();
foreach(@files) {
    $_ =~ /$root(_*)(\d+).*\.fits/;
    push(@chips,$2);
	
}


# Get parameters from the files
@lines = `dfits $regname | fitsort -d CRPIX1 CRPIX2 CRVAL1 CRVAL2 CD1_1 CD1_2 CD2_1 CD2_2`;
foreach (@lines) {
    @list = split(/\s+/,$_);

    push(@CRPIX1,$list[1]);
    push(@CRPIX2,$list[2]);
    push(@CRVAL1,$list[3]);
    push(@CRVAL2,$list[4]);
    push(@CD1_1,$list[5]);
    push(@CD1_2,$list[6]);
    push(@CD2_1,$list[7]);
    push(@CD2_2,$list[8]);
}

# Get the layout
@layout = ();
#($nx,$ny)=get_layout(\@layout);
#print "nxny $nx $ny \n";

($nx,$ny)=get_layout2(\@layout);
$nx++;
$ny++;

print "\n Mosaic geometry : nxny $nx $ny \n";

# $il = -1;
# foreach(@layout) {
#     $il++;
#     print "lay of ".$files[$il]." is : \n";
#     print "     ".$layout[$il]->[0]." and ".$layout[$il]->[1]."\n";
# }




# prepare the environment
my @xrange = ();
my @yrange = ();
&prepare_range(\@env_range,$nx,$ny);

@tmp = @{$env_range[1]->[1]};
print "@tmp \n";


# get the size of the total mosaic
$im0 = $files[0];
$line = `dfits $im0 | fitsort -d NAXIS1 NAXIS2`;
$line =~ /(\S*)\s+(\S*)\s+(\S*)/;
$naxis1 = $2;
$naxis2 = $3;
$size1 = $nx * $naxis1;
$size2 = $ny * $naxis2;


# Plot it
if ($size1/$size2 > 30/20) {
    $size = 280;
    @size = ($size,$size*$size2/$size1);
} else {
    $size = 200;
    @size = ($size*$size1/$size2,$size);
}
#$size = 4;
#@size = ($size,$size*$size2/$size1);
#print "size12 $size1 $size2 @size \n";

if (defined $noprint) {
    $win = PDL::Graphics::PGPLOT::Window->new({Dev=>'/xserve',Size=>\@size,Unit=>2});
} else {
    $dev = "\"".$out."\"/PS";
    print "DEV $dev \n";
    $win = PDL::Graphics::PGPLOT::Window->new({Dev=>$dev,Size=>\@size,Unit=>2});
}
# Loop on the chips

print "\n   Loop on the chips \n";
$ii = -1;
foreach (@layout) {
    $ii++;
    $chip = $chips[$ii];

    $name = $files[$ii];
    $layx = $layout[$ii]->[0];
    $layy = $layout[$ii]->[1];
    
    @range = @{$env_range[$layx]->[$layy]};
    print "lay $layx $layy range @range \n";

    # Plot fits file
    &plot_fits($win,$name,\@range,$bin,$chip,$conv);
    
    # Read reg file if given
    if (!defined $noreg) {
	
	# get name
	if (!defined $root_reg) { $root_reg = $root;}
	if ($root_reg !~ /_\Z/) {$root_reg .= "_";}
	if (!defined $ext_reg) {
	    $tmp = `basename \"$ext\" .fits`;
	    chomp $tmp;
	    $ext_reg = $tmp.".reg";
	} 

	$name = $root_reg.$chips[$ii].$ext_reg;
	print "name reg $name \n";

	# read it
	my (@lx,@ly);
	if (-e $name) {
	    &read_reg($name,\@lx,\@ly,$bin);
	}
    
	# plot it
	$ir = -1;
	foreach (@lx) {
	    $ir++;
	    my @x = @{$lx[$ir]};
	    my @y = @{$ly[$ir]};
	    push(@x,$x[0]);
	    push(@y,$y[0]);
	    print "plot reg @x @y \n";

	    $win->line(\@x,\@y,{PlotPosition => [$range->[0],$range->[1],$range->[2],$range->[3]]});
	}

    }
}
$win->close();



###############################################################
sub plot_fits {
    my ($win,$im,$range,$bin,$chip,$conv) = @_;

    my $array3 = rfits $im;

    if ($conv > 1) {
	$array = conv2d $array3, ones($conv,$conv), {Boundary => Reflect};
    } else {
	$array = $array3;
    }

    # get the size
    my $sizex = $array3->hdr->{NAXIS1};
    my $sizey = $array3->hdr->{NAXIS2};
    
    # rebin it 
    if ($bin > 1) {
	$sizex /= $bin;
	$sizey /= $bin;
	$sizex = int($sizex);
	$sizey = int($sizey);
	
	$array2 = rebin($array,$sizex,$sizey);
    } else {
	$array2 = $array;
    }

    $f = $array2->flat;
    my $nelem = nelem($f);
   
    # get the scale from a slice on the data
    if ($nelem > 2000000) {
	$s = $f->mslice([$nelem/2-1e6,$nelem/2+1e6]);
	$qs = $s->qsort;
    } else{
	$qs = $f->qsort;
    }
    $nelem = nelem($qs);
    
    $max = $qs->index($percent/100*$nelem);
    $min = $qs->index((1-$percent/100)*$nelem);

    print "min $min max $max size $sizex $sizey \n";

    
    $win->env(0, $sizex-1, 0, $sizey-1,1,-1,{PlotPosition => [$range->[0],$range->[1],$range->[2],$range->[3]]});
    $win->imag( $array2 ,$min,$max,{DrawWedge=>0,PlotPosition => [$range->[0],$range->[1],$range->[2],$range->[3]]});
    $win->text($chip,$sizex/3,$sizey/10,{colour=>0});

}
###############################################################
sub read_reg {
    my ($name,$lx,$ly,$bin) = @_;

    $ir = -1;
    open(REG,$name) || die "Impossible to open $name \n";
    while(<REG>) {
	$line = $_;
	if ($line !~ /polygon/ && $line !~ /POLYGON/) {next;}
	if($line =~ /\((.*)\)/) {
	    $ir++;
	    @list = split(/\,/,$1);
	    
	    $flag = 0;
	    foreach(@list) {
		if ($flag == 0) {
		    push(@{$lx->[$ir]},$_/$bin);
		    $flag = 1;
		} else {
		    push(@{$ly->[$ir]},$_/$bin);
		    $flag = 0;
		}
	    }
	}
    }

}
###############################################################
sub prepare_range {
    my ($env,$nx,$ny) = @_;
    my $i = 0;
    my $j = 0;
    
    $stepx = ((1-1/20/($nx-1))/$nx);
    $stepy = ((1-1/20/($ny-1))/$ny);

    $margx = 1/$marg/($nx-1);
    $margy = 1/$marg/($ny-1);

    for ($i=0;$i<$nx;$i++) {
	for ($j=0;$j<$ny;$j++) {
	    @{$env->[$i]->[$j]} = ($stepx*$i+($i*$margx),$stepx*($i+1)+(($i)*$margx),$stepy*$j+($j*$margy),$stepy*($j+1)+(($j)*$margy))
	    #@{$env->[$i]->[$j]} = (1/$nx*$i,1/$nx*($i+1),1/$ny*$j,1/$ny*($j+1));
	}
    }

}
###############################################################
sub get_layout {
    my ($lay) = @_;

    

    $lay->[0]->[0] = 0;
    $lay->[0]->[1] = 0;
    $lay->[1]->[0] = 1;
    $lay->[1]->[1] = 0;
    $lay->[2]->[0] = 0;
    $lay->[2]->[1] = 1;
    $lay->[3]->[0] = 1;
    $lay->[3]->[1] = 1;

    $nx = 2;
    $ny = 2;
    
    return ($nx,$ny);
}
###############################################################
sub get_layout2 {
    my ($lay) = @_;
    
    my $ii = -1;
    my @x1; my @y1; 
    foreach(@CD1_1) {
	$ii++;

	$px1 = (1-$CRPIX1[$ii]);
	$py1 = (1-$CRPIX2[$ii]);


	$x1 = $CRVAL1[$ii] + $CD1_1[$ii] * $px1 + $CD1_2[$ii] * $py1; 
	$y1 = $CRVAL2[$ii] + $CD2_1[$ii] * $px1 + $CD2_2[$ii] * $py1; 
	
	#print $files[$ii]." $x1 $y1 \n";
	push(@x1,$x1);
	push(@y1,$y1);
    }

    # group the lines ----------------------
    my @lindex=();
    group_line(\@y1,\@lindex);
    
    # Label the groups of lines  ---> @ylayout
    %data = ();
    $il = -1;
    foreach(@lindex) {
	$il++;
	$data{$il} = $_->{"M"};
    }
    @ind = keys(%data);
    @sorted = sort {$data{$a} <=> $data{$b}} @ind;
    
    my @ylayout = ();
    $ii = -1;
    foreach (@sorted) {
	$ii++;
	$ylayout[$_] = $ii;
    } 

    # Sort inside the lines
    my $ill = -1;
    foreach(@lindex) {
	$ill++;
	my @list_val = (); 
	my @list_ind = @{$_->{"I"}}; 
	foreach(@list_ind) {
	    push(@list_val,$x1[$_]);
	}

	#print "list_val @list_val \n";

	my %data = ();
	my $ii = -1;
	foreach (@list_val) {
	    $ii++;
	    $data{$list_ind[$ii]} = $_;
	}

	@sorted = sort {$data{$b} <=> $data{$a}} @list_ind;

	# Fill in layout
	$ylay = $ylayout[$ill];
	$ix = -1;
	foreach (@sorted) {
	    $ix++;
	    $lay->[$_]->[0] = $ix;
	    $lay->[$_]->[1] = $ylay;
	}
    }

    $ny = scalar(@lindex);
    $nx = scalar(@{$lindex[0]->{"L"}});
    return ($nx,$ny);
}
###############################################################
sub group_line {
    my ($y,$ind) = @_;

    my $tol = 1/10;
    my $it = -1;my $ig = 0;
    foreach(@$y) {
	$it++;
	my $yc = $_;
	#print "element $it $yc \n";
	
	my $flag = 1;
	foreach(@$ind) {
	    my $mean= $_->{"M"};
	    if (abs($yc - $mean) < $tol) {       # NEW ELEMENT
		#print "New element \n";
		push(@{$_->{"I"}},$it);
		push(@{$_->{"L"}},$yc);
		$_->{"M"} = mean(@{$_->{"L"}});
		$flag = 0;
		last;
	    }
	}
	if ($flag == 1) {                         # NEW GROUP
	    #print "New group \n";
	    $ind->[$ig]->{"M"} = $yc;
	    push(@{$ind->[$ig]->{"L"}},$yc);
	    push(@{$ind->[$ig]->{"I"}},$it);
	    $ig++;
	}
    }

    # foreach(@$ind) {
    # 	print "Group : ".$_->{"M"}."\n";
    # 	foreach(@{$_->{"I"}}) {
    # 	    print "$_ ";
    # 	}
    # 	print "\n";
    # }
   
}
###############################################################
sub mean {
    my @vals = @_;
    $mean = 0;
    foreach(@vals) {
	$mean += $_;
    }
    return $mean/scalar(@vals);
}
###############################################################

