#!/usr/bin/perl -w

#
# Update the Theli installation to add the STRACK
# Argument : reduce directory [default = scripts/${EXT}]
#

# 06.04.2009:
# I tidied up the code (TE)
#
# 08.04.2014:
# The program definitions for the stracktools are no longer just
# appended to the riginal progs.ini file. They are now inserted
# before the 'verbose script output flags' are possibly reset.

if ($#ARGV > -1) {
  $REDUCE_DIR = shift @ARGV;
} else {
  die "SYNOPSIS: to_update.pl reduce_dir";
}

print "Preparing installation";
die "PIPESOFT Variable no defined ... \n" if (!exists $ENV{"PIPESOFT"});
die "EXT Variable no defined ... \n"      if (!exists $ENV{"EXT"});
die "PIPE Variable no defined ... \n"     if (!exists $ENV{"PIPE"});

$PWD           = $ENV{"PWD"};
$EXT           = $ENV{"EXT"};
$PIPESOFT      = $ENV{"PIPESOFT"};
$STRACKDIR     = $PIPESOFT . "/stracktools";
$STRACKDIR_LIB = $PIPESOFT . "/stracktools/lib";

# delete an old installation if it exists:
system("rm -rf $STRACKDIR") if (-e $STRACKDIR);
system("mkdir $STRACKDIR");
system("mkdir -p $STRACKDIR_LIB/${EXT}") if (!-e "$STRACKDIR_LIB/${EXT}");

# Copy the source code
@list = (
  "fitslib",         "cphead",
  "filter",          "filter_seg",
  "hough1_1im_ref",  "getpix",
  "hough2_1im",      "hough_inter_ref",
  "hough_weigh_ref", "setpix"
);

foreach (@list) {
  system("cp  -r $PWD" . "/" . $_ . " $STRACKDIR \n");
}
system("cp -r $PWD" . "/Makefile $STRACKDIR \n");
system("cp -r $PWD" . "/conf $STRACKDIR \n");

# copy the scripts
system("cp -r $PWD/scripts $STRACKDIR \n");

# Compile the C++ programs
@list = (
  "fitslib",         "cphead",
  "filter",          "filter_seg",
  "hough1_1im_ref",  "getpix",
  "hough2_1im",      "hough_inter_ref",
  "hough_weigh_ref", "setpix"
);

print "Compiling C++ codes";

foreach (@list) {
  chdir $STRACKDIR . "/" . $_;
  system("make");
  system("make install");
}

# update the progs.ini
chdir $PWD;
print "Updating ${REDUCE_DIR}/progs.ini";
system("cp ${REDUCE_DIR}/progs.ini ./progs.ini.copy");
system("cp ${REDUCE_DIR}/progs.ini ${REDUCE_DIR}/progs.ini.copy");

# get rid of old STRACK tools definitions in the progs.ini file
# (if they exist):
open(INPUT,  "./progs.ini.copy");
open(OUTPUT, "> ./progs.ini.tmp");

while ($line = <INPUT>) {
  if ( $line !~ /STRACK/
    && $line !~ /P_CPHEAD/
    && $line !~ /P_SETPIX/
    && $line !~ /P_GETPIX/) 
  {
    print OUTPUT $line;
  }
}

close(INPUT);
close(OUTPUT);

# If an old stracktools block has been removed there might be many
# multiple empty lines in the file; get rid of them:
open(INPUT,  "./progs.ini.tmp");
open(OUTPUT, "> ./progs.ini.new");

$empty = 0;
while ($line = <INPUT>) {
  if ($line =~ /^ *$/) {
    if ($empty == 0) {
      print OUTPUT $line;
      $empty = 1;
    }
  } else {
    print OUTPUT $line;
    $empty = 0;
  }
}

close(INPUT);
close(OUTPUT);

# copy the scripts in the reduce directory
print "Copying stracktools scripts";
system("cp reduce/* $REDUCE_DIR");

# copy program definition of stracktools into the progs.ini file. We
# need to insert these definition before we possibly reset the script
# output verbose flag. For this we insert the contents before a
# comment line starting with '# restore verbose':
open(INPUT1, "./progs.ini.new");
open(INPUT2, "./scripts/progs.ini");
open(OUTPUT, "> ${REDUCE_DIR}/progs.ini");

while ($line1 = <INPUT1>) {
  if ($line1 =~ /# restore verbose/) {
    while ($line2 = <INPUT2>) {
      print OUTPUT $line2;
    }
  }
  print OUTPUT $line1;
}

close(OUTPUT);
close(INPUT1);
close(INPUT2);

# clean up and bye
print "Cleaning up!\n";
system("rm ./progs.ini.copy") if (-e "./progs.ini.copy");
system("rm ./progs.ini.tmp")  if (-e "./progs.ini.tmp");
system("rm ./progs.ini.new")  if (-e "./progs.ini.new");

print "All Done!\n";

exit 0;
